/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
dojo.provide("com.ibm.team.vvc.server.core.globalConfigurations.pickers.ContributionsPickerDialog");

dojo.require("dijit.form.Select");
dojo.require("dijit.form.Button");

dojo.require("jazz.ui.Dialog")
var Dialog = jazz.ui.Dialog;

dojo.require("com.ibm.team.vvc.shared.oslc.web.utils.CLMUtils");
var CLMUtils = com.ibm.team.vvc.shared.oslc.web.utils.CLMUtils;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController.VvcNotificationCenterController");
var NotificationCenterController = com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController.VvcNotificationCenterController;

dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory");
var ncFactory = com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory;

dojo.requireLocalization("com.ibm.team.vvc.server.core.globalConfigurations.pickers", "ContributionsPickerDialogMessages");

var gcIcon = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/globalconfiguration_mutable.gif');

/**
 * Creates a dialog which lets users select from a list of contributors.  
 */
dojo.declare("com.ibm.team.vvc.server.core.globalConfigurations.pickers.ContributionsPickerDialog", [dijit._Widget, dijit._Templated], {
	
	messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.globalConfigurations.pickers", "ContributionsPickerDialogMessages"),
	templatePath: dojo.moduleUrl("com.ibm.team.vvc.server.core", "globalConfigurations/pickers/templates/ContributionsPickerDialog.html"),
	widgetsInTemplate: true,
	
	ncc: null,
	
	NOT_SELECTED: "NOT_SELECTED",
	
	_dialog: null,
	_clmFriends: null,
	_selectedFriends: null,
	_oslcMessagesMap: {
		"oslc-resize" : "_handleOslcResize",
		"oslc-response" : "_handleOslcResponse",
	},
	
	_oslcResponseMap: {
		"oslc:results" : "_handleOslcResults"
	},
	
	postCreate: function(){
		this._createDialog();
		
		this.ncc = new NotificationCenterController();
		this.nccNode.appendChild(this.ncc.domNode);

		this.applicationPicker.set("disabled", true);
		
		// Hook up events
		this.closeButton.on("click", dojo.hitch(this, function(){
			this._dialog.close();
		}));
		
		this._dialog.on("close", dojo.hitch(this, function(){
			this.destroy();
		}));
		
		this.applicationPicker.on("change", dojo.hitch(this, this.onSelectApplication));
		window.addEventListener('message', dojo.hitch(this, this._onWindowMessage), false);
		this.ncc.on("sizechanged", dojo.hitch(this, this.refresh));
						
		var authNotification = ncFactory.createInfo({
			title: this.messages.authenticating,
			description: this.messages.authenticatingDescription
		}).dispatch();
		
		CLMUtils.authenticateToFriends().then(dojo.hitch(this, function(){
			authNotification.close();
		
			var loadingContributionsNotification = ncFactory.createInfo({
				title: this.messages.loadingContributions,
				description: this.messages.loadingContributionsDescription,
				closeOnEvent: {context: this, event: "refresh"}
			}).dispatch();
			
			this._populateApplicatonPicker();
		}));
		this.refresh();
	},
	
	_createDialog: function(){
		var dialogArgs = {
			primaryTitle: this.dialogTitle ? this.dialogTitle : this.messages.primaryTitle,
			iconPath: gcIcon,
			contentNode: this.domNode,
			destroyContents: true // Destroy everything when it closes
		}
		
		this._dialog = new Dialog(dialogArgs);
	},
	
	refresh: function(){	
		// Only need to adjust the width if our delegate UI is visible.
		if(dojo.hasClass(this.contentNode, "showDelegateUi")){
			this._dialog.setWidth(this.iframe.offsetWidth + 7 + "px");
		}
		this._dialog.updatePosition();
	},	
	
	_populateApplicatonPicker: function(){
		this.applicationPicker.set("disabled", true);
		
		var loadingNotification = ncFactory.createInfo({
			title: this.messages.loading,
			description: this.messages.loadingDescription
		}).dispatch();
		
		// Remove any existing options
		var options = this.applicationPicker.getOptions();
		this.applicationPicker.removeOption(options);
				
		this.applicationPicker.addOption({
			disabled: true,
			label: this.messages.selectAnApplication,
			selected: true,
			value: this.NOT_SELECTED
		});
		
		CLMUtils.findGcContributions().then(dojo.hitch(this, function(results){
			var options = [];
			for(var serverUrl in results){
				var server = results[serverUrl];
				for(var contributionUrl in server.contributions){
					var contribution = server.contributions[contributionUrl];
					var title = dojo.string.substitute(this.messages.pickerItemTitleTemplate, {
						applicationTitle: contribution.title,
						"server": server.serverPath
					});
					options.push({
						disabled: false,
						label: title,
						selected: false,
						value: contributionUrl
					});
				}
			}
			this.applicationPicker.addOption(options);
			this.applicationPicker.set("disabled", false);
			loadingNotification.close();
		}));
	},
		
	onSelectApplication: function(applicationUrl){
		var notification = ncFactory.createInfo({
			title: this.messages.loading,
			closeOnEvent: {context: this.iframe, event: "onload"}
		}).dispatch();
		dojo.removeClass(this.contentNode, "showDelegateUi");

		CLMUtils.getGcContributionSelectionDialogArgs(applicationUrl).then(dojo.hitch(this, function(dialogArgs){
			dojo.attr(this.iframe, "title", dialogArgs.title);
			dojo.attr(this.iframe, "src", dialogArgs.url + "#oslc-core-postMessage-1.0");
			dojo.style(this.iframe, "width", "100%");
			dojo.style(this.iframe, "height", 0);
			dojo.addClass(this.contentNode, "showDelegateUi");
			this.refresh();
		}));
	},
		
	/*
	 * The following 4 methods could be extracted into their own utility if
	 * they are needed in other classes.
	 */
	_onWindowMessage: function(event){
		// Hack to get around some Jazz sillyness
		var message = event.data;
		for(var key in this._oslcMessagesMap){
			if(message.indexOf(key) === 0){
				var data = message.substring(key.length + 1);
				var json = dojo.fromJson(data);
				var func = this._oslcMessagesMap[key];
				dojo.hitch(this, func, json)();
			}
		}
	},
		
	_handleOslcResponse: function(json){
		for(var key in this._oslcResponseMap){
			if(json[key]){
				var data = json[key];
				var func = this._oslcResponseMap[key];
				dojo.hitch(this, func, data)();
			}
		}
	},
	
	_handleOslcResize: function(json){
		dojo.style(this.iframe, "width", json["oslc:hintWidth"]);
		dojo.style(this.iframe, "height", json["oslc:hintHeight"]);
		this.refresh();
	},
	
	_handleOslcResults: function(json){
		this.ncc.deactivateNotificationCenters();
		if(json && json.length == 1){
			results = json[0]["rdf:resource"];
			this.onOk(results);
		} else {
			// Cannot select more than one...
			this.onCancel();
		}
		this._dialog.close();
	},
		
	/**
	 * Called when the user selects something in the dialog and presses the
	 * 'OK' button.
	 */
	onOk: function(result){
		// Stub
	},
	
	/**
	 * Called when the user clicks the cancel button.
	 */
	onCancel: function(){
		// Stub
	}
	
});
})();
