<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE mapper
    PUBLIC "-//mybatis.org//DTD Mapper 3.0//EN"
    "http://mybatis.org/dtd/mybatis-3-mapper.dtd">

<mapper namespace="ChangesetService">

  <select id="getChangeSetInfo" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT uri as changeset, title, description, created, creator, endPoint  
    FROM VVC_ChangeSet
    JOIN VVC_System ON VVC_System.id = VVC_ChangeSet.uri 
    WHERE VVC_ChangeSet.uri = #{changeset}
  </select>
  
  <select id="getChangesetFromChangesetConfiguration" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_ChangeSet.uri 
    FROM VVC_ChangeSet
    JOIN VVC_ConfResource ON VVC_ConfResource.point = VVC_ChangeSet.endPoint 
    WHERE VVC_ConfResource.uri = #{csConfig} 
  </select>
  
  <select id="findChangesetConfiguration" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_ConfResource.uri 
    FROM VVC_ConfResource
    JOIN VVC_ChangeSet ON VVC_ChangeSet.endPoint = VVC_ConfResource.point 
    WHERE VVC_ChangeSet.uri = #{csUri} 
  </select>
  
  <select id="getChangesetFromDimensionValue" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_ChangeSet.uri 
    FROM VVC_ChangeSet
    JOIN VVC_Point ON VVC_Point.uri = VVC_ChangeSet.endPoint  
    WHERE VVC_Point.changeSet = #{changeSetDimensionValue}  
  </select>
  
  <select id="findDirectChangeSets" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_ChangeSet.uri as changeset, title, description, created, creator, endPoint  
    FROM VVC_ChangeSet
    JOIN VVC_System on VVC_System.id = VVC_ChangeSet.uri
    JOIN VVC_Point ON VVC_Point.uri = VVC_ChangeSet.endPoint  
    WHERE VVC_Point.purpose = #{purpose} AND VVC_Point.configurationSpace = #{confSpace} AND VVC_Point.changeSet IS NULL   
  </select>
  
  <select id="findAllChangeSets" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_ChangeSet.uri as changeset, title, description, created, creator, endPoint  
    FROM VVC_ChangeSet
    JOIN VVC_System on VVC_System.id = VVC_ChangeSet.uri
    JOIN VVC_Point ON VVC_Point.uri = VVC_ChangeSet.endPoint  
    WHERE VVC_Point.purpose = #{purpose} AND VVC_Point.configurationSpace = #{confSpace}   
  </select>
  
  <select id="findDirectChangeSetsForUser" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_ChangeSet.uri as changeset, title, description, created, creator, endPoint  
    FROM VVC_ChangeSet
    JOIN VVC_System on VVC_System.id = VVC_ChangeSet.uri
    JOIN VVC_Point ON VVC_Point.uri = VVC_ChangeSet.endPoint  
    WHERE VVC_Point.purpose = #{purpose} AND VVC_Point.configurationSpace = #{confSpace} AND creator = #{user} AND VVC_Point.changeSet IS NULL  
  </select>
  
  <select id="findAllChangeSetsForUser" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_ChangeSet.uri as changeset, title, description, created, creator, endPoint  
    FROM VVC_ChangeSet
    JOIN VVC_System on VVC_System.id = VVC_ChangeSet.uri
    JOIN VVC_Point ON VVC_Point.uri = VVC_ChangeSet.endPoint  
    WHERE VVC_Point.purpose = #{purpose} AND VVC_Point.configurationSpace = #{confSpace} AND creator = #{user}  
  </select>
  
  <select id="findChangeSetMappings" parameterType="hashmap" resultType="hashmap">
    SELECT DISTINCT VVC_Mapping.uri as mapping, concept, point  
    FROM VVC_Mapping
    JOIN VVC_Point ON VVC_Point.uri = VVC_Mapping.point  
    WHERE VVC_Point.purpose = #{purpose} AND VVC_Point.changeSet = #{changeSet}  
  </select>
  
</mapper>
