CREATE TABLE VVC_SystemProps (
  name  			varchar(255) NOT NULL PRIMARY KEY,
  value				varchar(255) NOT NULL
)

CREATE TABLE VVC_System (
  id               varchar(255) NOT NULL PRIMARY KEY,
  creator          varchar(255) NOT NULL,
  created          {TIMESTAMP} NOT NULL,
  modified         {TIMESTAMP} NOT NULL,
  securityContext  varchar(255)
)

CREATE TABLE VVC_BinaryResource (
  id               varchar(255) NOT NULL PRIMARY KEY,
  data             {BLOB}
)

CREATE TABLE VVC_Counter (
  uri              varchar(255) NOT NULL PRIMARY KEY,
  etag             varchar(16) NOT NULL,
  counter          varchar(255)
)

CREATE INDEX VVC_Counter_ETag ON VVC_Counter (uri, etag)

CREATE TABLE VVC_ConfSpace (
  uri                  varchar(255) NOT NULL PRIMARY KEY,
  etag                 varchar(16) NOT NULL,
  title                {TITLE},
  description          {DESCRIPTION},
  dependencyResource   varchar(255)
)

CREATE INDEX VVC_ConfSpace_ETag ON VVC_ConfSpace (uri, etag)

CREATE TABLE VVC_ConfSpace_component (
  source	varchar(255) NOT NULL,
  target    {EXTERNALURI} NOT NULL,
  FOREIGN KEY (source) REFERENCES VVC_ConfSpace(uri) ON DELETE CASCADE
)

CREATE INDEX VVC_ConfSpace_component_s ON VVC_ConfSpace_component (source)

CREATE INDEX VVC_ConfSpace_component_t ON VVC_ConfSpace_component (target)

CREATE TABLE VVC_Dimension (
  uri              varchar(255) NOT NULL PRIMARY KEY,
  etag             varchar(16) NOT NULL,
  title            {TITLE},
  description      {DESCRIPTION},
  oneGreaterThan   varchar(255),
  range            varchar(255),
  namespacePrefix  varchar(255),
  namespaceUri     {EXTERNALURI}
)


CREATE TABLE VVC_ConfSpace_dimensions (
  source	varchar(255) NOT NULL,
  target    varchar(255) NOT NULL,
  FOREIGN KEY (source) REFERENCES VVC_ConfSpace(uri) ON DELETE CASCADE,
  FOREIGN KEY (target) REFERENCES VVC_Dimension(uri)
)

CREATE TABLE VVC_Point (
  uri                 varchar(255) NOT NULL PRIMARY KEY,
  etag                varchar(16) NOT NULL,
  configurationSpace  varchar(255) NOT NULL,
  timeOrder           {BIGINT},               # value of TimeOrder dimension property
  purpose             varchar(255),           # value of Purpose dimension property
  changeSet           varchar(255),            # value of ChangeSet dimension property
  FOREIGN KEY (configurationSpace) REFERENCES VVC_ConfSpace(uri)
)

CREATE TABLE VVC_Mapping (
  uri              varchar(255) NOT NULL PRIMARY KEY,
  etag             varchar(16) NOT NULL,
  concept          {EXTERNALURI} NOT NULL,
  version          {EXTERNALURI},
  point            varchar(255) NOT NULL,
  FOREIGN KEY (point) REFERENCES VVC_Point(uri)
)

CREATE INDEX VVC_ConfSpace_dimensions_s ON VVC_ConfSpace_dimensions (source)

CREATE INDEX VVC_ConfSpace_dimensions_t ON VVC_ConfSpace_dimensions (target)

CREATE TABLE VVC_Subspace (
  id                        varchar(255) NOT NULL PRIMARY KEY,
  owner                     varchar(255) NOT NULL,
  domain                    varchar(255),
  path                      varchar(255),
  FOREIGN KEY (owner) REFERENCES VVC_ConfSpace(uri) ON DELETE CASCADE
)

CREATE INDEX VVC_Dimension_ETag ON VVC_Dimension (uri, etag)

CREATE TABLE VVC_ChangeSet (
  uri              varchar(255) NOT NULL PRIMARY KEY,
  etag             varchar(16) NOT NULL,
  title            {TITLE},
  description      {DESCRIPTION},
  endPoint         varchar(255),
  minTimeOrder     {BIGINT},
  incomplete       varchar(255),
  changesetConcept varchar(255)
)

CREATE INDEX VVC_ChangeSet_ETag ON VVC_ChangeSet (uri, etag)

CREATE TABLE VVC_ChangeSet_changes (
  source	varchar(255) NOT NULL,
  target    varchar(255) NOT NULL,
  FOREIGN KEY (source) REFERENCES VVC_ChangeSet(uri) ON DELETE CASCADE,
  FOREIGN KEY (target) REFERENCES VVC_Mapping(uri)
)

CREATE INDEX VVC_ChangeSet_changes_s ON VVC_ChangeSet_changes (source)

CREATE INDEX VVC_ChangeSet_changes_t ON VVC_ChangeSet_changes (target)

CREATE TABLE VVC_ChangeSet_clmLinks (
  source	varchar(255)  NOT NULL,
  predicate	{EXTERNALURI} NOT NULL,
  target	{EXTERNALURI} NOT NULL,
  FOREIGN KEY (source) REFERENCES VVC_ChangeSet(uri) ON DELETE CASCADE
)

CREATE INDEX VVC_ChangeSet_clmLinks_s ON VVC_ChangeSet_clmLinks (source)



CREATE INDEX VVC_Point_ETag ON VVC_Point (uri, etag)

CREATE INDEX VVC_Point_configurationSpace ON VVC_Point (configurationSpace)

# Other (client-defined) properties
CREATE TABLE VVC_Point_properties (
  uri                  varchar(255) NOT NULL,
  pkey                 varchar(255),
  svalue               varchar(255),
  ivalue               {BIGINT},
  FOREIGN KEY (uri) REFERENCES VVC_Point(uri) ON DELETE CASCADE
)

CREATE INDEX VVC_Point_properties_pkey ON VVC_Point_properties (pkey)

CREATE TABLE VVC_ConfResource (
  uri              varchar(255) NOT NULL PRIMARY KEY,
  etag             varchar(16) NOT NULL,
  title            {TITLE},
  description      {DESCRIPTION},
  point            varchar(255) NOT NULL,
  base             varchar(255),
  mutable          {BOOLEAN} NOT NULL,
  archived         {BOOLEAN} NOT NULL,
  controlledBy     varchar(255),
  checksum         {HUGEINT} NOT NULL,
  checksumModified {TIMESTAMP} NOT NULL,
  FOREIGN KEY (point) REFERENCES VVC_Point(uri) 
)

CREATE INDEX VVC_ConfResource_ETag ON VVC_ConfResource (uri, etag)

CREATE INDEX VVC_ConfResource_point ON VVC_ConfResource (point)

CREATE TABLE VVC_ConfResource_selects (
  source	varchar(255) NOT NULL,
  target    varchar(255) NOT NULL,
  FOREIGN KEY (source) REFERENCES VVC_ConfResource(uri) ON DELETE CASCADE,
  FOREIGN KEY (target) REFERENCES VVC_Mapping(uri)
)

CREATE INDEX VVC_ConfResource_selects_s ON VVC_ConfResource_selects (source)

CREATE INDEX VVC_ConfResource_selects_t ON VVC_ConfResource_selects (target)

CREATE TABLE VVC_Dependency (
  uri              varchar(255) NOT NULL PRIMARY KEY,
  etag             varchar(16) NOT NULL
)

CREATE INDEX VVC_Dependency_ETag ON VVC_Dependency (uri, etag)

CREATE TABLE VVC_Dependency_dependencies (
  source	varchar(255) NOT NULL,
  target    varchar(255) NOT NULL,
  FOREIGN KEY (source) REFERENCES VVC_Dependency(uri) ON DELETE CASCADE,
  FOREIGN KEY (target) REFERENCES VVC_ConfResource(uri)
)

CREATE INDEX VVC_Dep_dependencies_s ON VVC_Dependency_dependencies (source)

CREATE INDEX VVC_Dep_dependencies_t ON VVC_Dependency_dependencies (target)

CREATE TABLE VVC_EnumerationValue (
  uri                                 varchar(255) NOT NULL PRIMARY KEY,
  etag                                varchar(16) NOT NULL,
  title                               {TITLE},
  dimension                           varchar(255) NOT NULL,
  oneGreaterThan                      varchar(255),
  maxEnumValueOfNextLesserDim         varchar(255),
  maxIntValueOfNextLesserDim          {BIGINT}
)

CREATE INDEX VVC_EnumerationValue_ETag ON VVC_EnumerationValue (uri, etag)

CREATE INDEX VVC_Mapping_ETag ON VVC_Mapping (uri, etag)

CREATE INDEX VVC_Mapping_Concept ON VVC_Mapping (concept)

CREATE TABLE VVC_Setting (
  uri              varchar(255) NOT NULL PRIMARY KEY,
  etag             varchar(16) NOT NULL,
  id               varchar(255),
  jsondata         {SETTING_JSONDATA}
)

CREATE INDEX VVC_Setting_ETag ON VVC_Setting (uri, etag)

CREATE TABLE VVC_CompareProvidersStorage (
  uri                      varchar(255) NOT NULL PRIMARY KEY,
  etag                     varchar(16) NOT NULL,
  providerServiceUri       {EXTERNALURI},
  requiresExplicitLocking  {BOOLEAN} NOT NULL
)

CREATE INDEX VVC_CmpProvidersStorage_ETag ON VVC_CompareProvidersStorage (uri, etag)

CREATE TABLE VVC_CompareProvider (
  id                          varchar(255) NOT NULL PRIMARY KEY,
  owner                       varchar(255) NOT NULL,
  title                       {TITLE},
  canProvideWebFunctionality  {BOOLEAN} NOT NULL,
  providerCompareWebUri       {EXTERNALURI},
  hasSeparateMergeProvider    {BOOLEAN} NOT NULL,
  providerMergeWebUri         {EXTERNALURI},
  ordering                    {INTEGER},
  uri                         {EXTERNALURI},
  
  FOREIGN KEY (owner) REFERENCES VVC_CompareProvidersStorage(uri) ON DELETE CASCADE
)

CREATE TABLE VVC_ScopeEventLog (
  uri                          varchar(255) NOT NULL PRIMARY KEY,
  etag                         varchar(16) NOT NULL,
  globalConfigurationListener  {BOOLEAN} NOT NULL,
  eventReceiverUrl             {EXTERNALURI} NOT NULL, #UNIQUE
  data                         {BLOB}
)

CREATE TABLE VVC_ScopeEventRestriction (
  eventLog                    varchar(255) NOT NULL,
  confSpace                   varchar(255) NOT NULL,
  
  FOREIGN KEY (eventLog) REFERENCES VVC_ScopeEventLog(uri) ON DELETE CASCADE,
  FOREIGN KEY (confSpace) REFERENCES VVC_ConfSpace(uri) ON DELETE CASCADE
)

CREATE INDEX VVC_ScopeEventLog_ETag ON VVC_ScopeEventLog (uri, etag)

CREATE TABLE VVC_GlobalConfiguration (
  uri              varchar(255) NOT NULL,
  etag             varchar(16) NOT NULL,
  title            {TITLE},
  description      {DESCRIPTION},
  mutable          {BOOLEAN} NOT NULL,
  controlledBy     varchar(255),
  version          {SMALLINT} NOT NULL,
  derivedFrom      {EXTERNALURI},
  PRIMARY KEY(uri, version)
)

CREATE INDEX VVC_GlobalConfiguration_ETag ON VVC_GlobalConfiguration (uri, version, etag)

CREATE TABLE VVC_GC_previousbaselines (
  source	varchar(255) NOT NULL,
  target    varchar(255) NOT NULL,
  version   {SMALLINT} NOT NULL,
  FOREIGN KEY (source, version) REFERENCES VVC_GlobalConfiguration(uri, version) ON DELETE CASCADE
)

CREATE TABLE VVC_GC_component (
  source	varchar(255) NOT NULL,
  target    {EXTERNALURI} NOT NULL,
  version   {SMALLINT} NOT NULL,
  FOREIGN KEY (source, version) REFERENCES VVC_GlobalConfiguration(uri, version) ON DELETE CASCADE
)

CREATE INDEX VVC_GC_component_s ON VVC_GC_component (source, version)

CREATE INDEX VVC_GC_component_t ON VVC_GC_component (target, version)

CREATE INDEX VVC_GC_previousbaselines_s ON VVC_GC_previousbaselines (source, version)

CREATE INDEX VVC_GC_previousbaselines_t ON VVC_GC_previousbaselines (target, version)

CREATE TABLE VVC_GC_contributions (
  source	varchar(255) NOT NULL,
  target    {EXTERNALURI} NOT NULL,
  version   {SMALLINT} NOT NULL,
  FOREIGN KEY (source, version) REFERENCES VVC_GlobalConfiguration(uri, version) ON DELETE CASCADE
)

CREATE INDEX VVC_GC_contributions_s ON VVC_GC_contributions (source, version)

CREATE INDEX VVC_GC_contributions_t ON VVC_GC_contributions (target, version)