/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){   
	
	dojo.provide("com.ibm.team.vvc.server.core.ui.configurations.AddConfigurationWizard");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

	dojo.require('dojo.i18n');
	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.configurations", "GlobalConfigurationCreationWizardMessages");
	
	dojo.require("jazz.ui.Wizard");
	dojo.require("jazz.ui.RemoteChooserResponder");
	dojo.require("jazz.app.auth");
		
	dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");

	dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
	var ContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
	
	dojo.require("com.ibm.team.vvc.server.core.ui.configurations.steps.AddToWhat");
	dojo.require("com.ibm.team.vvc.server.core.ui.configurations.steps.AddToExisting");
//	dojo.require("com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationSummary");
//	dojo.require("com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationProgressPage");
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController.VvcNotificationCenterController");
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.GlobalConfigurationResource");
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.GlobalConfigurationConstants");
	var GlobalConfigurationConstants = com.ibm.team.vvc.shared.core.ui.objects.GlobalConfigurationConstants;

	dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory")
	var ncFactory = com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory;
			
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;
	
	var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;

	dojo.declare("com.ibm.team.vvc.server.core.ui.configurations.AddConfigurationWizard", [dijit._Widget, dijit._Templated], { //FIXME call this globalconfigurationwizard
	
		templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/configurations/templates/AddConfigurationWizard.html'),
		messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.configurations", "GlobalConfigurationCreationWizardMessages"),
		widgetsInTemplate: true,

		addToExistingStep: null,
		addToWhatStep: null,
//		configurationSummaryStep: null,
//		configurationProgressPage: null,
		
		wizard: null,
		_storageKey: null,
		
		configurationUri: null,
		baseConfigurationUri: null,
		mutable: null,
		type: null,
		
		_destroys: null,
		_disconnects: null,
		
		notificationCenter: null,
		redirect: true,
		
		/*
		 * Even though this widget is built using the V2 protocols, for backwards compatibility with
		 * a V1 rich client, the V1 protocols are supported.
		 */
		_OSLC1_PROTOCOL_WINDOWNAME: '#oslc-windowName-1.0',
		_OSLC1_PROTOCOL_POSTMSG: '#oslc-postMessage-1.0',
		_OSLC2_PROTOCOL_WINDOWNAME: '#oslc-core-windowName-1.0',
		_OSLC2_PROTOCOL_POSTMSG:  '#oslc-core-postMessage-1.0',
		_OSLC_PROTOCOL: null,	// OSLC protocol being used
		_OSLC_PROTOCOL_VERSION: null,
		_remoteFrame: null,		// The remote frame to communicate with
					
		constructor: function(args){
			this._destroys = [];
			this._disconnects = [];
			this.notificationCenter = new com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController.VvcNotificationCenterController();
		},
		
		postCreate: function(){	
			// Initialize the storage services
			this._storageKey = "GlobalConfigurationCreationWizardStorageSpace"; // Add random (something) to guarantee unique keys
					
			// Get the URI's.  Multiple paths to pass the URI's in means we need to figure this out first
			if(location.search.length > 0){
				// Being created via URL, probably in a dialog

				var url = location.search;
				var query = dojo.queryToObject(url.substring(url.indexOf("?") + 1, url.length));
				
				if (query.redirect === "false") {
					this.redirect = false;
				}
				
				this.configurationUri = query.configurationUri;
				this.baseConfigurationUri = query.baseConfigurationUri;
				if (this.baseConfigurationUri) {
					this.mutable = query.mutable === "true";
				}
				this.type = query.type; 
			}
			
			// Critical, the steps use this object to communicate, although it is referred to as _data in most steps.
			var data = {
					input: {},
					newConfiguration: {}
			};
			
			// Load the configuration info.
			if(this.configurationUri){
				var configuration = ContextUtils.getConfiguration(this.configurationUri);

				data.input.configuration = {
						uri: configuration.getUri(),
						title: configuration.getTitle(),
						description: configuration.getDescription(), 
						type: configuration.getType()
				};
								
			}
			
			if(this.validParentTypes){
				data.input.validParentTypes = this.validParentTypes;
			}
			
			sessionStorage.setItem(this._storageKey, dojo.toJson(data));
									
			if(location.hash){
				this._OSLC_PROTOCOL = location.hash;
				
				// Set the version number
				var p = this._OSLC_PROTOCOL; // Keep the code clean
				if(p === this._OSLC1_PROTOCOL_WINDOWNAME || p === this._OSLC1_PROTOCOL_POSTMSG){
					this._OSLC_PROTOCOL_VERSION = 1;
				}
				if(p === this._OSLC2_PROTOCOL_WINDOWNAME || p === this._OSLC2_PROTOCOL_POSTMSG){
					this._OSLC_PROTOCOL_VERSION = 2;
				}
				
				this._remoteFrame = new jazz.ui.RemoteChooserResponder();
				this._destroys.push(this._remoteFrame);
			}
			
			/*
			 * Modern supported browsers (IE8+, and all mobile browsers) support the new HTML5
			 * storage mechanisms.  If the storage mechanisms are missing, then this 
			 * picker may not work properly, but presumably, the user is also not on
			 * a modern && supported browser.
			 */
			if(!sessionStorage){
				// We have a big problem, config wizard relies on session storage..
				console.err("Session storage is not present, this is problematic!  Attempting to continue...");
			}
			
			var title = this.messages.createGlobalConfiguration;
			
			// Start building the wizard up
			this.addToExistingStep = new com.ibm.team.vvc.server.core.ui.configurations.steps.AddToExisting({storageKey: this._storageKey});
//			TODO more wizard pages			
//			this.configurationSummaryStep = new com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationSummary({storageKey: this._storageKey});
//			this.configurationProgressPage = new com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationProgressPage({storageKey: this._storageKey});
			
			
			var stepsToAddToTheWizard = [];
			if (data.input && data.input.configuration) {
				this.addToWhatStep = new com.ibm.team.vvc.server.core.ui.configurations.steps.AddToWhat({storageKey: this._storageKey});
				stepsToAddToTheWizard.push(this.addToWhatStep);
			}
			stepsToAddToTheWizard.push(this.addToExistingStep);
//			TODO more wizard pages			
//			stepsToAddToTheWizard.push(this.configurationSummaryStep);
//			stepsToAddToTheWizard.push(this.configurationProgressPage);
			
			var wizardArgs = {
				fullPageContainer: this.wizardContentNode,
				steps: stepsToAddToTheWizard,
		        primaryTitle: title,
		        showStepSummary: true,
		        hideTrim: true,
		        includeCancel: true,
		        customProcessFinish: dojo.hitch(this, this.onFinish),
		        customProcessCancel: dojo.hitch(this, this.onCancel)
			}
			
			// The "step" arg will be injected by the dojo.hitch for _addConnectionsForStep
			dojo.forEach(wizardArgs.steps, dojo.hitch(this, this._addConnectionsForStep));
			
			// Make the wizard, 
			this.wizard = new jazz.ui.Wizard(wizardArgs);
			this.wizard.startup();
			this._destroys.push(this.wizard);

			this.informationCenterNode.appendChild(this.notificationCenter.domNode);
		},
		
		_addConnectionsForStep: function(step){
			this._disconnects.push(dojo.connect(step, "setSize", dojo.hitch(this, this._setFrameSize)));
			this._disconnects.push(dojo.connect(step, "showMessage", dojo.hitch(this, this.showMessage)));
			this._disconnects.push(dojo.connect(step, "stepAdded", dojo.hitch(this, this._addConnectionsForStep)));
			this._destroys.push(step);
		},
		
		destroy: function(args){
			this.inherited(args);
			dojo.forEach(this._disconnects, function(handle){dojo.disconnect(handle);});
			dojo.forEach(this._destroys, function(obj){dojo.destroyRecursive(obj);});
		},
		
		onFinish: function(){
			var data = dojo.fromJson(sessionStorage.getItem(this._storageKey));
			var args = {};
			
			if (data.addToExistingGlobalConfiguration) {
				var globalConfiguration = new com.ibm.team.vvc.shared.core.ui.objects.GlobalConfigurationResource();
				var link =/* VvcUtils.getVvcUri() + "/repository/configuration?uri=" + */data.addToExistingGlobalConfiguration.uri;
				var deferred = globalConfiguration.loadAsync(link, dojo.hitch(this, this.onLoadError));
				
				deferred.addCallback(this, function (param) {
					if (globalConfiguration.getContributions()) {
						globalConfiguration.getContributions().push({
							type: "uri",
							value: data.input.configuration.uri
						});
					} else {
						//ideally, serialization would include the empty json
						//node so I don't have to do this... 
						globalConfiguration.setContributions([data.input.configuration.uri]);
					}
					
					var deferred2 = globalConfiguration.doSaveAsync();
					deferred2.then(dojo.hitch(this, function (param) {
						//"data.addToExistingGlobalConfiguration.uri" should be the same as "param"
						var response = [dojo.toJson([{"rdf:resource": data.addToExistingGlobalConfiguration.uri}])];
						this._sendOslcResponse(response);
						if (this.redirect) {
							//don't use objectToQuery for simple strings
							window.parent.location.href = VvcUtils.getVvcUri() + "/web?contextUri=" + encodeURIComponent(data.addToExistingGlobalConfiguration.uri) + "#action=editGlobalConfiguration";
						}
					}), dojo.hitch(this, function(error) {
						ncFactory.createXhrError({xhrError: error, ioArgs: error.response.options.ioArgs}).dispatch();
					}));
				});
				
				//args.addToExistingGlobalConfiguration = data.addToExistingGlobalConfiguration;
			} else {
				var newGlobalConfigurationDetails = data.addToNewGlobalConfiguration;
				if (data.input.configuration) {
					newGlobalConfigurationDetails.contributions = [data.input.configuration.uri];
				}
				
				if (this.baseConfigurationUri) {
					this._populateNewConfigurationFromBaseConfiguration(newGlobalConfigurationDetails);
				}
				
				var newGlobalConfiguration = new com.ibm.team.vvc.shared.core.ui.objects.GlobalConfigurationResource(newGlobalConfigurationDetails);

				var deferred = newGlobalConfiguration.doCreate();
				deferred.then(dojo.hitch(this, function (param) {
					//"data.addToExistingGlobalConfiguration.uri" should NOT be the same as "param" because it was https://tobeassigned
					var response = [dojo.toJson([{"rdf:resource": param}])];
					this._sendOslcResponse(response);
					if (this.redirect) {
						//don't use objectToQuery for simple strings
						window.parent.location.href = VvcUtils.getVvcUri() + "/web?contextUri=" + encodeURIComponent(param) + "#action=editGlobalConfiguration";
					}
				}), dojo.hitch(this, function(error) {
					ncFactory.createXhrError({xhrError: error, ioArgs: error.response.options.ioArgs}).dispatch();
				}));
				
				//args.addToNewGlobalConfiguration = data.addToNewGlobalConfiguration;
			}
		},
		
		_populateNewConfigurationFromBaseConfiguration: function(newGlobalConfigurationDetails) {
			var baseConfiguration = ContextUtils.getConfiguration(this.baseConfigurationUri);
			newGlobalConfigurationDetails.mutable = this.mutable;
			newGlobalConfigurationDetails.component = baseConfiguration.getProperty(GlobalConfigurationConstants.GC_COMPONENT);
			var previousBaselinesProperty = baseConfiguration.getArrayProperty(GlobalConfigurationConstants.GC_PREVIOUS_BASELINES);
			var previousBaselines;
			if (previousBaselinesProperty) {
				previousBaselines = [];
				for (var i = 0; i < previousBaselinesProperty.length; i++) {
					previousBaselines.push(previousBaselinesProperty[i].value);
				}
			}
			if (!baseConfiguration.getBooleanProperty(GlobalConfigurationConstants.GC_MUTABLE)) {
				if (!previousBaselines) {
					previousBaselines = [this.baseConfigurationUri];
				} else {
					previousBaselines.push(this.baseConfigurationUri);
				}
			}
			newGlobalConfigurationDetails.previousBaselines = previousBaselines;
		},
		
		onCancel: function(){
			// Respond with a "cancelled" event.  An empty array for OSLC:Response represents a "cancel" event.
			this._sendOslcResponse();	// Sends the cancel event
		},
		
		_sendOslcResponse: function(results){
			if (this._remoteFrame) {
			
				// Default to sending a cancel event
				if(!results){
					results = "[]";
				}
										
				var response = {};
				
				if(this._OSLC_PROTOCOL_VERSION === 1){			
					response["oslc_vc:message"] = "oslc_vc.create";
					response["oslc_vc:results"] = results;
				} else /*if(this._OSLC_PROTOCOL_VERSION === 2)*/ {	// Uncomment if we get to OSLC v3
					response["oslc:message"] = "oslc.create";
					response["oslc:results"] = results;
				}
				
				this._remoteFrame.sendResponse(dojo.toJson(response));
			}
		},
		
		_setFrameSize: function(size){
			if (this._remoteFrame) {
				var response = {};
				
				if(this._OSLC_PROTOCOL_VERSION === 1){			
					response["oslc_vc:message"] = "window.size";
					response["oslc_vc:size"] = size;
				} else /*if(this._OSLC_PROTOCOL_VERSION === 2)*/ {	// Uncomment if we get to a new version of OSLC
					response["oslc:message"] = "window.size";
					response["oslc:size"] = size;
				}
				
				this._remoteFrame.sendResponse(dojo.toJson(response));
			}
		}
				
	});
	
})();