/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){   
	
	dojo.provide("com.ibm.team.vvc.server.core.ui.configurations.ConfigurationCreationWizard");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

	dojo.require('dojo.i18n');
	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.configurations", "ConfigurationCreationWizardMessages");
	
	dojo.require("jazz.ui.Wizard");
	dojo.require("jazz.ui.RemoteChooserResponder");
	dojo.require("jazz.app.auth");
		
	dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");

	dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
	var ContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
	
	dojo.require("com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationDetails");
	dojo.require("com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationBase");
	dojo.require("com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationSummary");
	dojo.require("com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationProgressPage");
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationSpace");
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource");
	dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController.VvcNotificationCenterController");
			
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;
	
	var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;

	dojo.declare("com.ibm.team.vvc.server.core.ui.configurations.ConfigurationCreationWizard", [dijit._Widget, dijit._Templated], {
	
		templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/configurations/templates/ConfigurationCreationWizard.html'),
		messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.configurations", "ConfigurationCreationWizardMessages"),
		widgetsInTemplate: true,
		
		wizard: null,
		configurationTitleStep: null,
		baseConfigurationStep: null,
		configurationSummaryStep: null,
		configurationProgressPage: null,
		_storageKey: null,
		
		configurationUri: null,
		spaceUri: null,
		type: null,
		vvcInternal: null,
		
		_destroys: null,
		_disconnects: null,
		
		notificationCenter: null,
		
		/*
		 * Even though this widget is built using the V2 protocols, for backwards compatibility with
		 * a V1 rich client, the V1 protocols are supported.
		 */
		_OSLC1_PROTOCOL_WINDOWNAME: '#oslc-windowName-1.0',
		_OSLC1_PROTOCOL_POSTMSG: '#oslc-postMessage-1.0',
		_OSLC2_PROTOCOL_WINDOWNAME: '#oslc-core-windowName-1.0',
		_OSLC2_PROTOCOL_POSTMSG:  '#oslc-core-postMessage-1.0',
		_OSLC_PROTOCOL: null,	// OSLC protocol being used
		_OSLC_PROTOCOL_VERSION: null,
		_remoteFrame: null,		// The remote frame to communicate with
					
		constructor: function(args){
			this._destroys = [];
			this._disconnects = [];
			this.notificationCenter = new com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController.VvcNotificationCenterController();
		},
		
		postCreate: function(){	
			// Initialize the storage services
			this._storageKey = "ConfigurationCreationWizardStorageSpace"; // Add random (something) to guarantee unique keys
					
			// Get the URI's.  Multiple paths to pass the URI's in means we need to figure this out first
			if(location.search.length > 0){
				// Being created via URL, probably in a dialog

				var query = dojo.queryToObject(location.search.substring(1));
				
				this.configurationUri = query.configurationUri;
				this.spaceUri = query.spaceUri;
				this.type = query.type; 
				this.validParentTypes = query.validParentTypes;
			}/* else {
				// Deal with other ways to pass in parent configuration info.  
				// NOTE: At this point in the widget lifecycle, if the URI's were passed in via a mixin, 
				// 	they are already taken care of.
			}*/
			
			// Critical, the steps use this object to communicate, although it is referred to as _data in most steps.
			var data = {
				input: {},
				newConfiguration: {},
				vvcInternal: query.vvcInternal === "true",
			};
			
			// Load the configuration info.
			if(this.configurationUri){
				var configuration = ContextUtils.getConfiguration(this.configurationUri);

				data.input.configuration = {
						uri: configuration.getUri(),
						title: configuration.getTitle(),
						description: configuration.getDescription(), 
						type: configuration.getType()
				};
								
				// We can get away with only passing in the configuration because the space info is implictly provded
				if(!this.spaceUri){
					this.spaceUri = configuration.getSpaceUri();
				}				
			}
			
			if(this.validParentTypes){
				data.input.validParentTypes = this.validParentTypes;
			}
			
			// Load the space info.  The space info is the bare-minimum required
			if(this.spaceUri){
				var space = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationSpace();
				this._destroys.push(space);
				space.load(this.spaceUri);
				
				data.input.space = {
						uri: space.getUri(),
						title: space.getTitle(),
						description: space.getDescription(),
						type: space.getType()
				};
			}
							
			// Type *must* be specified.
			data.input.type = this.type;
			
			sessionStorage.setItem(this._storageKey, dojo.toJson(data));
									
			if(location.hash){
				this._OSLC_PROTOCOL = location.hash;
				
				// Set the version number
				var p = this._OSLC_PROTOCOL; // Keep the code clean
				if(p === this._OSLC1_PROTOCOL_WINDOWNAME || p === this._OSLC1_PROTOCOL_POSTMSG){
					this._OSLC_PROTOCOL_VERSION = 1;
				}
				if(p === this._OSLC2_PROTOCOL_WINDOWNAME || p === this._OSLC2_PROTOCOL_POSTMSG){
					this._OSLC_PROTOCOL_VERSION = 2;
				}
				
				this._remoteFrame = new jazz.ui.RemoteChooserResponder();
				this._destroys.push(this._remoteFrame);
			}
			
			/*
			 * Modern supported browsers (IE8+, and all mobile browsers) support the new HTML5
			 * storage mechanisms.  If the storage mechanisms are missing, then this 
			 * picker may not work properly, but presumably, the user is also not on
			 * a modern && supported browser.
			 */
			if(!sessionStorage){
				// We have a big problem, config wizard relies on session storage..
				console.err("Session storage is not present, this is problematic!  Attempting to continue...");
			}
			
			var title = data.newConfiguration.type === vvcConstants.WORKSPACE_TYPE ? this.messages.createWorkspace : this.messages.createSnapshot;
			
			// Start building the wizard up
			this.configurationDetailsStep = new com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationDetails({storageKey: this._storageKey});
			this.baseConfigurationStep = new com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationBase({storageKey: this._storageKey});
			this.configurationSummaryStep = new com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationSummary({storageKey: this._storageKey});
			this.configurationProgressPage = new com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationProgressPage({storageKey: this._storageKey});
			var wizardArgs = {
				fullPageContainer: this.wizardContentNode,
				steps: [this.configurationDetailsStep,
				        this.baseConfigurationStep,
				        this.configurationSummaryStep,
				        this.configurationProgressPage],
		        primaryTitle: title,
		        showStepSummary: true,
		        hideTrim: true,
		        includeCancel: true,
		        customProcessFinish: dojo.hitch(this, this.onFinish),
		        customProcessCancel: dojo.hitch(this, this.onCancel)
			}
			
			// The "step" arg will be injected by the dojo.hitch for _addConnectionsForStep
			dojo.forEach(wizardArgs.steps, dojo.hitch(this, this._addConnectionsForStep));
			
			// Make the wizard, 
			this.wizard = new jazz.ui.Wizard(wizardArgs);
			this.wizard.startup();
			this._destroys.push(this.wizard);			

			this.informationCenterNode.appendChild(this.notificationCenter.domNode);
		},
		
		_addConnectionsForStep: function(step){
			this._disconnects.push(dojo.connect(step, "setSize", dojo.hitch(this, this._setFrameSize)));
			this._disconnects.push(dojo.connect(step, "showMessage", dojo.hitch(this, this.showMessage)));
			this._disconnects.push(dojo.connect(step, "stepAdded", dojo.hitch(this, this._addConnectionsForStep)));
			this._destroys.push(step);
		},
		
		destroy: function(args){
			this.inherited(args);
			dojo.forEach(this._disconnects, function(handle){dojo.disconnect(handle);});
			dojo.forEach(this._destroys, function(obj){dojo.destroyRecursive(obj);});
		},
		
		onFinish: function(){
			var data = dojo.fromJson(sessionStorage.getItem(this._storageKey));
			
			var response = [];
			var args = {};
			if(data.actions.load){
				args.load = data.actions.load;
			}
			
			if(data.actions.add){
				args.add = data.actions.add;
			}
			
			response.push(dojo.toJson(args));
			this._sendOslcResponse(response);
		},
		
		onCancel: function(){
			// Respond with a "cancelled" event.  An empty array for OSLC:Response represents a "cancel" event.
			this._sendOslcResponse();	// Sends the cancel event
		},
		
		// TODO: Investigate more elegant solution to the following OSLC stuff.
		_sendOslcResponse: function(results){
			
			// Default to sending a cancel event
			if(!results){
				results = "[]";
			}
									
			var response = {};
			
			if(this._OSLC_PROTOCOL_VERSION === 1){			
				response["oslc_vc:message"] = "oslc_vc.create";
				response["oslc_vc:results"] = results;
			} else /*if(this._OSLC_PROTOCOL_VERSION === 2)*/ {	// Uncomment if we get to OSLC v3
				response["oslc:message"] = "oslc.create";
				response["oslc:results"] = results;
			}
			
			this._remoteFrame.sendResponse(dojo.toJson(response));
		},
		
		_setFrameSize: function(size){			
			var response = {};
			
			if(this._OSLC_PROTOCOL_VERSION === 1){			
				response["oslc_vc:message"] = "window.size";
				response["oslc_vc:size"] = size;
			} else /*if(this._OSLC_PROTOCOL_VERSION === 2)*/ {	// Uncomment if we get to a new version of OSLC
				response["oslc:message"] = "window.size";
				response["oslc:size"] = size;
			}
			
			this._remoteFrame.sendResponse(dojo.toJson(response));
		}
				
	});
	
})();