/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide("com.ibm.team.vvc.server.core.ui.explorer.VvcExplorerTable");

dojo.require('dojo.i18n');
dojo.require("dijit._Widget");

dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;

dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;

dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.VvcButtonFactory");
var vvcButtonFactory = com.ibm.team.vvc.server.core.ui.internal.util.VvcButtonFactory;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.ConfigurationUiNode");
var ConfigurationUiNode = com.ibm.team.vvc.shared.web.ui.util.ConfigurationUiNode;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.GlobalConfigurationUiNode");
var GlobalConfigurationUiNode = com.ibm.team.vvc.shared.web.ui.util.GlobalConfigurationUiNode;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.ConfigurationSpaceUiNode");
var ConfigurationSpaceUiNode = com.ibm.team.vvc.shared.web.ui.util.ConfigurationSpaceUiNode;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.ControllingAreaUiNode");
var ControllingAreauiNode = com.ibm.team.vvc.shared.web.ui.util.ControllingAreaUiNode;

dojo.require("com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGridTable");

dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.explorer", "VvcExplorerMessages");
	
dojo.declare("com.ibm.team.vvc.server.core.ui.explorer.VvcExplorerTable", [com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGridTable], {
	
	messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.explorer", "VvcExplorerMessages"),
	
	/**
	 * Registers all the row builders with this table.  The order in which they are registered will also reflect
	 * the order in which the columns appear.
	 */
	registerRowBuilders: function(){
		builders = [];
		// Order matters!
		builders.push(this._actionsBuilder());
		builders.push(this._nameBuilder());
		builders.push(this._controllingAreaBuilder());
		return builders;
	},
		
	/**
	 * Builds the actions column for a row
	 */
	_actionsBuilder: function(tr, data, table){
		var builder = {
				columnName: this.messages.actionsColumnTitle,
				columnClass: "actionsColumn",
				builderFunc: dojo.hitch(this, function(tr, data, table){
					var readyDeferred = new dojo.Deferred();
					var superReadyDeferred = this._defaultBuilder(tr, data, table);
					dojo.when(superReadyDeferred, dojo.hitch(this, function(td){					
						var gc = tr._gc;
						
						//we don't support actions for gloabl confiurations for now
						if (data.GLOBAL) {
							readyDeferred.resolve(td);
							return;
						}
						
						var toolbar = new jazz.ui.Toolbar({
							"class": "actionsToolbar",
							onfocus: dojo.hitch(this, function(){tr.onFocus()}),
							onblur: dojo.hitch(this, function(){tr.onBlur()})
						});
						gc.destroys(toolbar);
						
						var actionsBar = new jazz.ui.Toolbar({
							"class": "actionMenuPopout",
						});
						gc.destroys(actionsBar);
						
						var actionsButton = new jazz.ui.toolbar.MenuButton({
							menuProvider: function(){
								return actionsBar;
							},
							cacheMenu: true,
							title: "Actions",
							imgClass: "icons-show-actions",
							"class": "actionsToolbarMenuButton",
						});
						gc.destroys(actionsButton);
						
						gc.disconnects(dojo.connect(actionsButton, "open", this, function(){
							dojo.setAttr(tr.domNode, "menuOpen", null);
						}));
						
						gc.disconnects(dojo.connect(actionsButton, "close", this, function(){
							dojo.removeAttr(tr.domNode, "menuOpen");
						}));

						toolbar.addChild(vvcButtonFactory.getCheckboxButton(data, {
							actionCallback: dojo.hitch(this, function(args){
								args.checked ? tr.onSelect() : tr.onDeselect();
							})
						}));
						actionsBar.addChild(vvcButtonFactory.getCreateWorkspaceButton(data, {actionCallback: dojo.hitch(this, function(){
							if(data.type === "workspace"){
								tr._isExpanded = true;
								tr.onRefresh();
							} else if(data.type === "snapshot"){
								var row = tr.childOf ? tr.childOf : tr;
								row.onRefresh();
							}
						})}));
						actionsBar.addChild(vvcButtonFactory.getCreateSnapshotButton(data, {actionCallback: dojo.hitch(this, function(){
							data.type === "snapshot" ? tr.childOf.onRefresh() :  tr.onRefresh();						
						})}));
						actionsBar.addChild(new jazz.ui.toolbar.Separator());
						actionsBar.addChild(vvcButtonFactory.getArchiveUnarchiveConfigurationButton(data, {actionCallback: dojo.hitch(this, function(){table.onRefresh()})}));
						actionsBar.addChild(vvcButtonFactory.getProjectAssociationButton(data, {actionCallback: dojo.hitch(this, function(){table.onRefresh()})}));
						actionsBar.addChild(new jazz.ui.toolbar.Separator());
						actionsBar.addChild(vvcButtonFactory.getRefreshButton(null,{actionCallback: dojo.hitch(this, function(){tr.onRefresh()})}, null));
						toolbar.addChild(actionsButton);
						td.appendChild(toolbar.domNode);
						readyDeferred.resolve(td);
					}));
					return readyDeferred;
				})
		};
		return builder;
	},
	
	/**
	 * Builds the name column for the row
	 */
	_nameBuilder: function(tr, data, table){
		childIndent = 20; //px to indent each child level by
		var builder = {
				columnName: this.messages.nameColumnTitle,
				columnClass: "configurationNameColumn",
				builderFunc: dojo.hitch(this, function(tr, data, table){
					var readyDeferred = new dojo.Deferred();
					var superReadyDeferred = this._defaultBuilder(tr, data, table);
					dojo.when(superReadyDeferred, dojo.hitch(this, function(td){
						var gc = tr._gc;
						// Consume space before the title
						if(tr._childLevel){
							var childIndenter = dojo.create("span", {
								// Directly setting the style here is a secessary evil since we need to generate the width.
								style: "width:" + (childIndent * tr._childLevel) + "px;display:inline-block;",
								innerHTML: "&nbsp;"
							});
							td.appendChild(childIndenter);
						}
						
						// Add the expando, or a placeholder to keep everything neatly lined up
						if(data.children && data.children.length > 0){
							var expando = new com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGridTableExpando({
								explorerTable: this,
								tableRow: tr
							});
							td.appendChild(expando.domNode);
							gc.destroys(expando);
						} else {
							var expandoHolder = dojo.create("span", {
								"class": "vvcExplorerExpando",
								innerHTML: "&nbsp;",
							});
							td.appendChild(expandoHolder)
						}
						
						var configuratonTitleNode = data.GLOBAL ? new GlobalConfigurationUiNode({
							uri: data.uri
						}, td) : new ConfigurationUiNode({
							uri: data.uri
						}, td);
						
						gc.destroys(configuratonTitleNode);
						
						gc.disconnects(dojo.connect(configuratonTitleNode.domNode, 'onfocus', dojo.hitch(this, function(){
							tr.onFocus();
						})));
						gc.disconnects(dojo.connect(configuratonTitleNode.domNode, 'onblur', dojo.hitch(this, function(){
							tr.onBlur();
						})));
						
						gc.disconnects(dojo.connect(configuratonTitleNode, 'onfocus', dojo.hitch(this, function(){
							tr.onFocus();
						})));
						gc.disconnects(dojo.connect(configuratonTitleNode, 'onblur', dojo.hitch(this, function(){
							tr.onBlur();
						})));
						
						readyDeferred.resolve(td);
					}));
					return readyDeferred;
				})
		};
		return builder;
	},
	
	/**
	 * Builds the controlling area column for the row
	 */
	_controllingAreaBuilder: function(tr, data, table){
		var builder = {
				columnName: this.messages.controllingAreaColumnTitle,
				columnClass: "controllingAreaColumn",
				builderFunc: dojo.hitch(this, function(tr, data, table){
					var readyDeferred = new dojo.Deferred();
					var superReadyDeferred = this._defaultBuilder(tr, data, table);
					dojo.when(superReadyDeferred, dojo.hitch(this, function(td){
						if(!data.GLOBAL){
							var gc = tr._gc;
							var controllingAreaNode = new ControllingAreauiNode({
								uri: data.controllingAreaUri,
								title: data.controllingAreaName,
							}, td);
							gc.destroys(controllingAreaNode);
							
							gc.disconnects(dojo.connect(controllingAreaNode.domNode, 'onfocus', dojo.hitch(this, function(){
								tr.onFocus();
							})));
							gc.disconnects(dojo.connect(controllingAreaNode.domNode, 'onblur', dojo.hitch(this, function(){
								tr.onBlur();
							})));
						}
						readyDeferred.resolve(td);
					}));
					return readyDeferred;
				})
		};
		return builder;
	}		
});
})();
