/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide("com.ibm.team.vvc.server.core.ui.internal.changesets.ChangesetsExplorer");

dojo.require('dojo.i18n');
dojo.require("dijit._Widget");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("jazz.ui.toolbar.Separator");

dojo.require("com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGrid");

dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.VvcButtonFactory");
var buttonFactory = com.ibm.team.vvc.server.core.ui.internal.util.VvcButtonFactory;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.require("com.ibm.team.vvc.server.core.ui.internal.changesets.ChangesetsExplorerTable");

dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.changesets", "ChangesetsPageMessages");

dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory");
var ncFactory = com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory;

dojo.declare("com.ibm.team.vvc.server.core.ui.internal.changesets.ChangesetsExplorer", [com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGrid], {
	  
	messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.changesets", "ChangesetsPageMessages"),
		
	_configurationUri: null,
	_user: null,
	_showOnlyActiveChangesets: null,
	
	/**
	 * <p>Creates a new change set explorer</p>
	 * <p>"@param" refers to 'arg' properties.</p>
	 * 
	 * @param configrationUri 
	 * 				(required) Show change sets for this configuration
	 */
	constructor: function(args){
		// Required
		this._configurationUri = args.configurationUri;
		this._user = null,
		this._showOnlyActiveChangesets = true;
	},
				
	buildTitleNode: function(){
		// Empty node
		return dojo.create("span");
	},
	
	buildActionsNode: function(){
		var toolbar = new jazz.ui.Toolbar();
		
		var createChangesetButton = buttonFactory.getCreateChangesetButton(null, {
			actionCallback: dojo.hitch(this, this.refresh)
		});
		toolbar.addChild(createChangesetButton);
		
		toolbar.addChild(new jazz.ui.toolbar.Separator());
		
		var showUsersChangesetsOnlyButton = buttonFactory.getToggleChangesetsShowUserOrShowAllButton(null, {
			actionCallback: dojo.hitch(this, function(){
				this._user = showUsersChangesetsOnlyButton.pressed ? "*" : null;
				this.refresh();
			})
		});
		toolbar.addChild(showUsersChangesetsOnlyButton);
		
		var showActiveChangeSetsOnlyButton = buttonFactory.getToggleShowOnlyActiveChangesetsButton(null, {
			actionCallback: dojo.hitch(this, function(){
				this._showOnlyActiveChangesets = !showActiveChangeSetsOnlyButton.pressed;
				this.refresh();
			})
		});
		toolbar.addChild(showActiveChangeSetsOnlyButton);
		
		toolbar.addChild(new jazz.ui.toolbar.Separator());
		
		var refreshButton = buttonFactory.getRefreshButton(null, {
			actionCallback: dojo.hitch(this, this.refresh)
		});
		toolbar.addChild(refreshButton);
		
		return toolbar.domNode;
	},
	
	buildTreeGridTableNode: function(){
		this._gridTable = new com.ibm.team.vvc.server.core.ui.internal.changesets.ChangesetsExplorerTable();
		return this._gridTable.domNode;
	},
	
	refresh: function(){	
		
		this._gridTable.refresh();
		this._idRowMap = [];
		
		var content = {
			includeCreated: true,
			configuration: this._configurationUri,
			user: this._user
		};
		
		if(this._showOnlyActiveChangesets){
			content.filterCommitted = true;
		}

		var def = jazz.client.xhrGet({
			handleAs: 'json',
			url: VvcUtils.getVvcUri() + "/changesets/query?" + dojo.objectToQuery(content), //$NON-NLS-1$
		});
		
		def.then(dojo.hitch(this, function(results){
			for(var index = 0; index < results.length; index++){
				this._gridTable.addRow(results[index]);
			}
		}), dojo.hitch(this, function(error){
			ncFactory.createXhrError({xhrError: error, ioArgs: error.response.options.ioArgs}).dispatch();
		}));
	},
			
	onRefreshRowById: function(id){
		// Cannot refresh row by id, refresh the whole table instead.
		this.refresh();
	},
	
	onExpandRow: function(row, data, childOf){	
		var urlParams = {
			changeset: data.uri,
			withResources: true,
			includeDisplayNames: true,
			includePermissions: false,
			configuration: this._configurationUri
		}
		
		var getResourcesDeferred = jazz.client.xhrGet({
			url: VvcUtils.getVvcUri() + "/changesets/queryOne?" + dojo.objectToQuery(urlParams), //$NON-NLS-1$
		});
		
		getResourcesDeferred.then(dojo.hitch(this, function(results){
			if(results){
				var jsonResults =  dojo.fromJson(results);
				if(jsonResults.length > 0){
					var changeset = jsonResults[0];
					var resources = changeset.resources;
					if(resources.length > 0){
						for(var i = 0; i < resources.length; i++){
							this._gridTable.addRow(resources[i], row);
						}
						return;
					}
				}
			}
			
			// Show the "no results" node if there are no results
			this._gridTable.addRow({showNoResults: true}, row);
		}), dojo.hitch(this, function(error){
			ncFactory.createXhrError({xhrError: error, ioArgs: error.response.options.ioArgs}).dispatch();
		}));
	}	
});
})();
