/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.vvc.server.core.ui.internal.configurations.ConfigurationCompactRendering");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.util.iframe");
dojo.require("jazz.client.xhr");
dojo.require("com.ibm.team.vvc.shared.core.ui.objects.Constants");
var Constants = com.ibm.team.vvc.shared.core.ui.objects.Constants;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.ConfigurationUiNode");
var ConfigurationUiNode = com.ibm.team.vvc.shared.web.ui.util.ConfigurationUiNode;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.ConfigurationSpaceUiNode");
var ConfigurationSpaceNode = com.ibm.team.vvc.shared.web.ui.util.ConfigurationSpaceUiNode;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.UserUiNode");
var UserUiNode = com.ibm.team.vvc.shared.web.ui.util.UserUiNode;

dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource");

dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.configurations", "ConfigurationCompactRenderingMessages");

dojo.declare("com.ibm.team.vvc.server.core.ui.internal.configurations.ConfigurationCompactRendering", [dijit._Widget, dijit._Templated], {

	messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.configurations", "ConfigurationCompactRenderingMessages"),
	
	templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/internal/configurations/templates/ConfigurationCompactRendering.html'),
	
	postCreate: function() {
		this.inherited(arguments);
		
		if (location.search.length > 0) {
		var obj = dojo.queryToObject(location.search.substring(1));
		if (obj) {
			if (obj.configurationUri) {
				var context1 = {
					messages: this.messages,
					self: this
				};
			
				var parentConfig = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
				
				//if you are trying to access or debug from an independent ajax module, 
				//don't use VvcUtils.getVvcUri() because BootstrapProperties.FRONTSIDE_URL is not set
				//in that case, please use the commented code to obtain the contextRoot
				//var contextRootIndex = obj.configurationUri.lastIndexOf("/configurations/");
				//var contextRoot = obj.configurationUri.substring(0, contextRootIndex);
				var link = VvcUtils.getVvcUri() + "/repository/configuration?uri=" + encodeURIComponent(obj.configurationUri);
				var deferred1 = parentConfig.loadAsync(link);

				deferred1.then(dojo.hitch(context1, function(configuration, ioArgs) {
						
						if (!configuration) {
							dojo.addClass(this.self.tableNode, "configuration-compact-rendering-hidden");
							dojo.addClass(this.self.errorNode, "configuration-compact-rendering-visible");
							return;
						}
						
						try {
							var description = configuration.getDescription();
							if (!description || description.length == 0) {
								description = this.messages.empty;
							}
							this.self.descriptionNode.appendChild(dojo.doc.createTextNode(description));
						} catch (err) {
							this.self.descriptionNode.appendChild(dojo.doc.createTextNode(this.messages.unknown));
						}
						
						var configurationDetails = [];
						if (configuration.isSnapshot()) {
							configurationDetails["configurationType"] = this.messages.configurationTypeSnapshot;
						} else if (configuration.isWorkspace()) {
							configurationDetails["configurationType"] = this.messages.configurationTypeWorkspace;
						} else if (configuration.isChangeset()) {
							configurationDetails["configurationType"] = this.messages.configurationTypeChangeset;
						} else {
							configurationDetails["configurationType"] = this.messages.configurationTypeUnknown;
						}
						
						this.self.descriptiveNode.appendChild(dojo.doc.createTextNode(dojo.string.substitute(this.messages.descriptive, configurationDetails)));
						
						try {
							var userUiNode = new UserUiNode({
								uri: configuration.getCreator(),
								includeLink: false,
								includeHover: false
							});
							this.self.creatorNode.appendChild(userUiNode.domNode);
						} catch (err) {
							this.self.creatorNode.appendChild(dojo.doc.createTextNode(this.messages.unknown));
						}

						try {
							var created = new Date(configuration.getCreated());
							created = created.toLocaleDateString() + " " + created.toLocaleTimeString();
							this.self.createdNode.appendChild(dojo.doc.createTextNode(created));
						} catch (err) {
							this.self.createdNode.appendChild(dojo.doc.createTextNode(this.messages.unknown));
						}
						
						try {
							var configurationSpaceNode = new ConfigurationSpaceNode({
								uri: configuration.getSpaceUri()
							});
							this.self.spaceNode.appendChild(configurationSpaceNode.domNode);
						} catch (err) {
							this.self.spaceNode.appendChild(dojo.doc.createTextNode(this.messages.unknown));
						}
						

						try {
							var parentWorkspaceUri = configuration.getParentWorkspaceUri();
							var parentSnapshotUri = configuration.getParentSnapshotUri();
							
							if(configuration.getType() === "snapshot"){
								dojo.empty(this.self.parentSnapshotRow);
								dojo.attr(this.self.parentWorkspaceLabel, "innerHTML", this.messages.parentWorkspace);
								var parentWorkspaceContent;
								if(parentWorkspaceUri){
									parentWorkspaceContent = (new ConfigurationUiNode({
										uri: parentWorkspaceUri,
										includeHover: false,
										includeLink: false
									})).domNode;
								} else {
									parentWorkspaceContent = dojo.doc.createTextNode(this.messages.empty);
								}
								this.self.parentWorkspaceNode.appendChild(parentWorkspaceContent);
							} else if(configuration.getType() === "workspace") {
								dojo.attr(this.self.parentWorkspaceLabel, "innerHTML", this.messages.parentFlowTarget);
								var parentWorkspaceContent;
								if(parentWorkspaceUri){
									parentWorkspaceContent = (new ConfigurationUiNode({
										uri: parentWorkspaceUri,
										includeHover: false,
										includeLink: false
									})).domNode;
								} else {
									parentWorkspaceContent = dojo.doc.createTextNode(this.messages.empty);
								}
								this.self.parentWorkspaceNode.appendChild(parentWorkspaceContent);
								
								dojo.attr(this.self.parentSnapshotLabel, "innerHTML", this.messages.parentSnapshot);
								var parentSnapshotContent;
								if(parentSnapshotUri){
									parentSnapshotContent = (new ConfigurationUiNode({
										uri: parentSnapshotUri,
										includeHover: false,
										includeLink: false
									})).domNode;
								} else {
									parentSnapshotContent = dojo.doc.createTextNode(this.messages.empty);
								}
								this.self.parentSnapshotNode.appendChild(parentSnapshotContent);
							} /* else something is wrong*/ 
						} catch (err) {
							parentUri = undefined;
							this.self.parentLabel.appendChild(dojo.doc.createTextNode(this.messages.parent));
							this.self.parentNode.appendChild(dojo.doc.createTextNode(this.messages.unknown));
						}						
					}),
					dojo.hitch(this, function(responseerror) {
						dojo.addClass(this.self.tableNode, "configuration-compact-rendering-hidden");
						dojo.addClass(this.self.errorNode, "configuration-compact-rendering-visible");
					}));
					
				} else {
					dojo.addClass(this.tableNode, "configuration-compact-rendering-hidden");
					dojo.addClass(this.errorNode, "configuration-compact-rendering-visible");
				}
			}
		
		}
		
		if (window.postMessage && window.parent) {
			var newPreviewHeight = jazz.util.iframe.getDocumentHeight();
			var resizeMessage = "oslc-preview-height:" + newPreviewHeight;
			window.parent.postMessage(resizeMessage, "*");
			resizeMessage = "jazz-compact-rendering-preview-height:" + newPreviewHeight;
			window.parent.postMessage(resizeMessage, "*");
		} 
	}
	
});

})();
