/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {

	dojo.provide("com.ibm.team.vvc.server.core.ui.internal.deliver.DeliverUtil");
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.Constants");
	var Constants = com.ibm.team.vvc.shared.core.ui.objects.Constants;
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.DeliverSession");
	var DeliverSession = com.ibm.team.vvc.shared.core.ui.objects.DeliverSession;

	/**
	 * Deliver utility that centralizes the logic for launching the deliver UI.
	 */
	com.ibm.team.vvc.server.core.ui.internal.deliver.DeliverUtil = {
		
		/**
		 * Starts the deliver UI. Requires one object argument with the following keys:
		 * 
		 * 1. sourceUri
		 *    The source URI of a configuration you want to deliver or compare from.
		 *    In the case of rebase, this is the snapshot that you want to act as the new
		 *    Base Snapshot of the workspace.
		 *    
		 * 2. targetUri
		 *    The target URI of a configuration you want to deliver or compare to.
		 *    In the case of rebase, this is the workspace for which you want to change
		 *    the base snapshot of.
		 *    
		 * 3. refererUri
		 *    The URI to which the user will be taken to after the deliver session
		 *    is completed or discarded.
		 *    
		 * 4. goToStep2
		 *    Boolean value to indicate that the deliver UI should go to step 2 (change
		 *    set picker) directly when it launches.
		 *    
		 * 5. operation
		 *    One of 
		 *      (a) Constants.DELIVER_OPERATION_DELIVER (to deliver from source to target)
		 *      (b) Constants.DELIVER_OPERATION_COMPARE (to compare source / target)
		 *      (c) Constants.DELIVER_OPERATION_REBASE (to rebase target (workspace) to source (snapshot))
		 *      
		 * 6. errorFunc
		 *    A function (response, ioArgs) that will be called in the case an error occurs.
		 */
		startDeliverUi: function(args) {
			if (!args) {
				args = {};
			}
			
			var sourceUri = args.sourceUri;
			var targetUri = args.targetUri;			
			var refererUri = args.refererUri;
			
			var session = new DeliverSession();
			session.setState(Constants.DELIVER_SESSION_STATE_UNINITIALIZED);
			
			if (sourceUri) {
				session.setSourceUri(sourceUri);
			}
			
			if (targetUri) {
				session.setTargetUri(targetUri);
			}
			
			if (refererUri) {
				session.setRefererUrl(refererUri);
			}
			
			var goToStep2 = false;
			if (args.goToStep2 && sourceUri && targetUri) {
				goToStep2 = true;
			}
			
			if (args.operation) {
				session.setOperation(args.operation);
				
				// force going to step 2 if this is a rebase operation.
				if (args.operation === Constants.DELIVER_OPERATION_REBASE) {
					goToStep2 = true;
				}
				
			} else {
				session.setOperation(Constants.DELIVER_OPERATION_DELIVER);
			}
			
			var vvcUri = VvcUtils.getVvcUri();
			vvcUri += Constants.DELIVER_FRONT_SERVICE_PATH;
			
			session.doPostAsync(vvcUri, dojo.hitch(this, function(response, ioArgs) {
				this._onSuccess(response, ioArgs, goToStep2);
			}), args.errorFunc);
		},
		
		_onSuccess: function(response, ioArgs, step2) {
			if (ioArgs) {
				var xhr = ioArgs.xhr;
				if (xhr) {
					var location = xhr.getResponseHeader("Location");
					if (location) {
						location = location.replace(Constants.DELIVER_FRONT_SERVICE_PATH, Constants.DELIVER_UI_FRONT_SERVICE_PATH);
						if(dojo.config.isDebug){
							location += "?debug=true";
						}
						if (step2) {
							location += "#/steps/2";
						} else {
							location += "#/steps/1";
						}
						
						document.location.href = location;
					}
				}
			}
		},
		
		/**
		* Provides the default compare (and merge) web Uri
		*/
		getDefaultCompareMergeWebUri: function() {
			return VvcUtils.getVvcUri() + "/vvc-compare?embed=true";
		}
		
	};

})();
