/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){

	dojo.provide("com.ibm.team.vvc.server.core.ui.internal.page.ResourceMappingViewer");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("jazz.ui.Toolbar");
	dojo.require("jazz.ui.toolbar.Button");
	dojo.require("jazz.ui.StyledTable");
	
	dojo.require("net.jazz.ajax.ui.PlatformUI");
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;

	dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
	var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
	var ContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
	var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;

	dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.page", "ResourceMappingViewerMessages");
	var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.page", "ResourceMappingViewerMessages");
	
	var VVC_APP = "com.ibm.team.vvc.server.core";

	dojo.declare("com.ibm.team.vvc.server.core.ui.internal.page.ResourceMappingViewer", [dijit._Widget, dijit._Templated], {

		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl(VVC_APP, "ui/internal/page/templates/ResourceMappingViewer.html"),

		_configurationUri: null,
		_lazyStart: false,
		_mappings: null,
		_showActions: true,
		_hasChanges: null,
		selectUri: null,

		constructor: function(args) {
		if (args) {
			if (args.configurationUri) {
				this._configurationUri = args.configurationUri;
				this._lazyStart = args.lazyStart;
				if (args.showActions === false) {
					this._showActions = args.showActions;
				}
			}
		}
		this.messages = messages;
	},

	postCreate: function() {
		if (!this._lazyStart) {
			this.load();
		}
	},

	load: function(resizableContainer) {
		// show loading message
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: this.messages.loading
		});
		
		var link = this.selectUri ? this.selectUri : VvcUtils.getVvcUri() + "/repository/configuration/resourceVersions?configurationUri=" + encodeURI(this._configurationUri);
		
		var args = {
			url: link,
			headers: {
				"Accept" : "application/json;application/rdf+json;text/rdf+json;text/json"
			},
			handleAs: "json"
		};

		// build resource version table
		var deferred = jazz.client.xhrGet(args);
		deferred.addCallback(this, function (param) {
			var tableContent = this._buildContentForResourceTab(param, resizableContainer);
		});
		deferred.addErrback(this, function(error) {
			this.showMessage(PageUtil.createXhrErrorMessage(deferred.ioArgs));
		});
		
		return deferred;
	},
	
	_buildContentForResourceTab: function(mappings, resizableContainer) {
		this._mappings = mappings;
		
		// remove loading message
		this.hideMessage();

		// find deltas in the mappings
		if (mappings) {
			for (var i = 0; i < mappings.length; i++) {
				var entry = mappings[i];
				if (entry.deltaType) {
					this._hasChanges = true;
					break;
				}
			}
		}
		
		// create table header
		var columnData = [];
		if (this._hasChanges === true) {
			columnData.push( { label: "", width: "16px" } );
		}
		columnData.push( { label: this.messages.name, sortable: true } );
		columnData.push( { label: this.messages.version, width: "100px", sortable: true } );
		if (this._showActions === true) {
			columnData.push({ label: this.messages.actions, width: "100px" });
		}
		var container = this._containerNode;
		if (this.tableContainer) {
			container.removeChild(this.tableContainer);
		}

		// create table contents
		this.tableContainer = dojo.create("div", {style: "overflow: auto; white-space: nowrap; display: inline-block"}, container);
		var self = this;
		table = new jazz.ui.StyledTable({
			trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
			columns: columnData,
			sortBy: 0,
			sortByAscending: false,
			handleSort: function() {
				this.clearBody();
				self._sort(mappings, table);
				self._fillTable(mappings, table);
			}
		});
		this._resourceMappingTable = table;
		this.tableContainer.appendChild(table.domNode);
		table.domNode.style.width = "100%";
		
		dojo.addClass(this.tableContainer, "vvc-editor-divTableContentStyle");
		if (resizableContainer) {
			resizableContainer.addResizableChildElement(this.tableContainer);
		}
		
		// change version ui from version number
		if (mappings) {
			var msg = dojo.string.substitute(this.messages.resourceCount, [mappings.length]);
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: msg
			});

			for (var i = 0; i < mappings.length; i++) {
				var entry = mappings[i];
				
				// initialize name field
				entry.name = entry.displayName;
				if (!entry.name) {
					if (entry.concept.indexOf( VvcUtils.getVvcUri() + "/dependencies/space/") !== -1) {
						entry.name = this.messages.dependencies;
					} else {
						entry.name = entry.concept;
					}
				}
				
				// change version to last segment
				entry.versionId = entry.version;
				if (entry.versionId) {
					var index = entry.versionId.lastIndexOf("/");
					if (index != -1) {
						entry.versionId = entry.versionId.substring(index + 1);
					}
				}
			}
		}
		
		this._sort(mappings, table);
		this._fillTable(mappings, table);
	},
	
	_fillTable: function(mappings, table) {
		// Now add the rows
		if (mappings && mappings.length) {
			var outgoingAddIcon, outgoingChangeIcon, incomingDeleteIcon, incomingChangeIcon, conflictIcon;
			var tr, td, label;
			for (var i = 0; i < mappings.length; i++){
				var entry = mappings[i];
				tr = document.createElement("tr");
				
				if (this._hasChanges === true) {
					td = document.createElement("td");
			        var img = dojo.create("IMG", null, td);
					if (entry.deltaType === "ChangeDelta") {
				        if (!outgoingChangeIcon) {
				        	outgoingChangeIcon = dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/outchange.gif');
				        }
				        img.src = outgoingChangeIcon;
					} else if (entry.deltaType === "AddDelta") {
				        if (!outgoingAddIcon) {
				        	outgoingAddIcon = dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/outadd.gif');
				        }
				        img.src = outgoingAddIcon;
					} else if (entry.deltaType === "DeleteDelta") {
				        if (!incomingDeleteIcon) {
				        	incomingDeleteIcon = dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/indelete.gif');
				        }
				        img.src = incomingDeleteIcon;
					} else if (entry.deltaType === "OtherContributorChangeDelta") {
				        if (!incomingChangeIcon) {
				        	incomingChangeIcon = dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/inchange.gif');
				        }
				        img.src = incomingChangeIcon;
					} else if (entry.deltaType === "ChangeChangeConflict") {
				        if (!conflictIcon) {
				        	conflictIcon = dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/conflict.gif');
				        }
				        img.src = conflictIcon;
					}
					tr.appendChild(td);
				}

				// name column
				td = document.createElement("td");
				var name = entry.name;
				if (entry.displayNameHref) {
					this.setLinkField(td, name, entry.displayNameHref);
				} else {
					this.setLinkField(td, name);
				}
				tr.appendChild(td);
				
				// version column
				td = document.createElement("td");
				this.setLinkField(td, entry.versionId);
				tr.appendChild(td);
				
				// resource actions column
				if (this._showActions === true) {
					td = document.createElement("td");
					var tb = this._createActions(entry, td);
					td.appendChild(tb.domNode);
					tr.appendChild(td);
				}
				
				table.tbody.appendChild(tr);
			}
		}

		return table;
	},
	
	setLinkField: function(field, value, uri) {
		if (field) {
			if (field.firstChild) {
				field.removeChild(field.firstChild);
			}

			if (uri) {
				var link = dojo.create("a");
				link.href = uri;
				link.innerHTML = jazz.util.html.escape(value);
				field.appendChild(link);
			} else {
				var node = dojo.doc.createTextNode(value);
				field.appendChild(node);
			}
		}
	},
	
	_sort: function(configResources, table) {
		var tableColumn = this._getTableColumn;
		var mutableIndex = this._mutableIndex;
		var f = function(a, b) {
			var c1 = tableColumn(a, table.sortBy, mutableIndex);
			var c2 = tableColumn(b, table.sortBy, mutableIndex);
			var r = 0;
			if (c1 && c2) {
				if (c1 < c2) {
					r = -1; 
				} else if (c1 > c2) {
					r = 1;
				}
				if (table.sortByAscending) {
					r = r * -1;
				}
			} else if (c1) {
				return -1;
			} else {
				return 1;
			}
			return r;
		};
		configResources.sort(f);
	},

	_getTableColumn: function(mapping, columnIndex, index) {
		if (columnIndex === 0) {
			return mapping.name;
		}
		if (columnIndex === 1) {
			return mapping.versionId;
		}
	},
	
	showMessage: function(args) {
		if (args && this._messageBoxContainer) {
			args["closable"] = true;
			if (!this.messageBox) {
				this.messageBox = new jazz.ui.MessageBox();
				this._messageBoxContainer.appendChild(this.messageBox.domNode);
			}
			this.messageBox.showMessage(args);
			this._messageBoxContainer.style.display = "";
		}
	},
	
	hideMessage: function() {
		if (this._messageBoxContainer) {
			this._messageBoxContainer.style.display = "none";
		}
	},
	
	_actionRegistry: [
	{
		label:  messages.history,
		imgSrc: dojo.moduleUrl("com.ibm.team.vvc.shared.web", "graphics/history.gif"),
		handler: function (resourceEntry) {
			var query = {};
			query.contextUri = ContextUtils.getContextUri();
			query.conceptUri = resourceEntry.concept;
			if (resourceEntry.displayName) {
				query.resourceDisplayName = resourceEntry.displayName;
				if (resourceEntry.displayNameHref) {
					query.resourceDisplayNameHref = resourceEntry.displayNameHref;
				} else if (resourceEntry.displayNameHref !== undefined) {
					query.resourceDisplayNameHref = "";
				}
			}
			
			// create breadcrumb parameter
			var config = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
			var deferred = config.load(query.contextUri);
			var title = config.getTitle();
			if (title) {
				query.breadcrumb = title;
			}

			PageUtil.showPage(vvcConstants.SHOW_HISTORY, dojo.objectToQuery(query));
		}
	}],
	
	_createActions: function(resourceEntry, td) {
		var toolbar = new jazz.ui.Toolbar();
		var actions = this._actionRegistry;
		dojo.forEach(actions, dojo.hitch(this, function (action)
		{
			var button = jazz.ui.toolbar.Button({
				title: action.label,
				imgSrc: action.imgSrc,
				onClick: dojo.hitch(this, function(){
					action.handler.call(this, resourceEntry, td);
				})
			});
			button.setAttribute("enabled", true);
			toolbar.addChild(button);
		}));
		dojo.addClass(toolbar.domNode, "vvc-resources-toolbar");		
		return toolbar;
	},

	})   
})();
