/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("com.ibm.team.vvc.server.core.ui.internal.util.VvcButtonFactory");
dojo.require("jazz.ui.toolbar.Button");
dojo.require("jazz.ui.toolbar.TwoStateButton");

dojo.require("com.ibm.team.vvc.shared.web.ui.util.TeamAreaDialogContent");

dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;

dojo.provide("com.ibm.team.vvc.shared.core.ui.objects.Constants");
var Constants = com.ibm.team.vvc.shared.core.ui.objects.Constants;

dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory");
var ncFactory = com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.SecurityConstants");
var SecurityConstants = com.ibm.team.vvc.shared.web.ui.util.SecurityConstants;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.SecurityServiceUtils");
var SecurityServiceUtils = com.ibm.team.vvc.shared.web.ui.util.SecurityServiceUtils;

dojo.require("com.ibm.team.vvc.shared.web.access.AccessUtils");
var AccessUtils = com.ibm.team.vvc.shared.web.access.AccessUtils;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.InfoContentUtils");
var InfoContentUtils = com.ibm.team.vvc.shared.web.ui.util.InfoContentUtils;

dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.require("com.ibm.team.vvc.server.core.ui.internal.deliver.DeliverUtil");
var DeliverUtil = com.ibm.team.vvc.server.core.ui.internal.deliver.DeliverUtil;

dojo.require("com.ibm.team.vvc.shared.web.ui.configurations.CreateConfigurationForm");
var CreateConfigurationForm = com.ibm.team.vvc.shared.web.ui.configurations.CreateConfigurationForm;

dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
var ContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.LinkToWorkItemUtil");
var LinkToWorkItemUtil = com.ibm.team.vvc.shared.web.ui.util.LinkToWorkItemUtil;

dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.util", "VvcButtonFactoryMessages");
var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.util", "VvcButtonFactoryMessages");

dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.context", "ConfigurationContextMessages");
var configurationContextMessages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.context", "ConfigurationContextMessages");

com.ibm.team.vvc.server.core.ui.internal.util.VvcButtonFactory = {
	
		/**
		 * A factory for creating buttons useful in VVC.
		 * 
		 * General rundown of parameters:
		 * 
		 * @param data 	(optional) The data available at button-creation time.  For example, the Create Workspace 
		 * 				button uses this parameter to extract the configuration which the new workspace
		 * 				will be based on, since this information is available when the button is created.
		 * 
		 * @param options 	(optional) These are options which affect how the button is to be created.  For example, 
		 * 					a Checkbox Button might be created and, using the options parameter, it will
		 * 					be created as ticked
		 * 
		 *  @param onClikcArgs 	(optional) Arguments which are only available when the button has been clicked.  
		 *  					Argument should be a function which is hitched back into the calling context.  
		 *  					For example, the Compare Button button uses this to determine the selected 
		 *  					configurations 
		 */
		
		// -------------------------------------------------------------------------------
		
		getCompareButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-compare",
				title: (options && options.title) ? options.title : messages.compareButtonTitle,
				onClick: dojo.hitch(this, function(){
					// onClickArgs are required!
					var args = data ? data : {};
					args.operation = Constants.DELIVER_OPERATION_COMPARE;
					if(onClickArgs){
						// onClick args override create-time args
						dojo.mixin(args, onClickArgs());
					}
					
					args.errorFunc = function(error, ioArgs) {
						ncFactory.createXhrError({xhrError: error, ioArgs: args}).dispatch();
					};
					args.refererUri = window.location.href;
					
					DeliverUtil.startDeliverUi(args);
				}),
			};
			var applicable = function(button){
				if(data && data.underConstruction){
					return false;
				}
				return !(data) || (data.ancestor && data.descendant) && (data.ancestor.archive ? ! data.ancestor.archive : true) || (data.descendant ? data.descendant.archive : true); 
			};
			return this._makeButton(args, options, applicable);
		},
		
		getDeliverButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-deliver",
				title: messages.deliverButtonTitle,
				onClick: dojo.hitch(this, function(){
					// onClickArgs are required!
					var args = data ? data : {};
					
					if(onClickArgs){
						// onClick args override create-time args
						dojo.mixin(args, onClickArgs());
					}
					
					args.errorFunc = function(error, ioArgs) {
						ncFactory.createXhrError({xhrError: error, ioArgs: args}).dispatch();
					};
					args.refererUri = window.location.href;

					DeliverUtil.startDeliverUi(args);
				})
			};
			var applicable = function(button){
				if(data && data.underConstruction){
					return false;
				}
				return true;
			};
			var button = this._makeButton(args, options, applicable);
			return button;
		},
		
		getCreateWorkspaceButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-create-workspace",
				title: messages.createWorkspaceButtonTitle,
				onClick: dojo.hitch(this, function(){
					var form = new CreateConfigurationForm({
						type: "workspace",
						configurationUri: data.uri,
						spaceUri: data.spaceUri,
						vvcInternal: true,
						onClose: dojo.hitch(this, function(value){
							if(options && options.actionCallback){
								options.actionCallback(value);
							}
						})
					});
				})
			};
			var applicable = function(button){
				if(data && data.underConstruction){
					return false;
				}
				if(!data.archive){
					return data.allowedActions[SecurityConstants.permissions.createOrModifyWorkspace];
				}
				return false;
			};
			return this._makeButton(args, options, applicable);
		},
		
		getCreateSnapshotButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-create-snapshot",
				title: messages.createSnapshotButtonTitle,
				onClick: dojo.hitch(this, function(){
					var form = new CreateConfigurationForm({
						type: "snapshot",
						configurationUri: data.uri,
						spaceUri: data.spaceUri,
						vvcInternal: true,
						onClose: dojo.hitch(this, function(value){
							if(options && options.actionCallback){
								options.actionCallback(value);
							}
						})
					});
				})
			};
			var applicable = function(button){
				if(data && data.underConstruction){
					return false;
				}
				if(!data.archive && data.mutable == true){
					return data.allowedActions[SecurityConstants.permissions.createOrModifySnapshot];
				}
				return false;
			};
			return this._makeButton(args, options, applicable);
		},

		getArchiveUnarchiveConfigurationButton: function(data, options, onClickArgs){
			var args = {
				imgClass: data.archive ? "icons-unarchive" : "icons-archive",
				title: data.archive ? messages.unarchiveButtonTitle : messages.archiveButtonTitle,
				onClick: dojo.hitch(this, function(){
					var result;
					var args = {
						url: VvcUtils.getVvcUri() + "/configurations/" + (data.archive === true ? "un" : "") + "archive?configuration=" + data.uri,
						handleAs: "json",
						headers: {
							Accept: "application/json;application/rdf+json;text/rdf+json;text/json"
						}
					};
					var deferred = jazz.client.xhrPost(args);
					var notification;
					deferred.then(function(param){
						if(options && options.actionCallback){
							options.actionCallback(param);
						}
						if(notification){
							notification.close();
						}
					}, /*Error Handler */function(err){
						/*
						 * Archiving gets a special error message because we have
						 * to alert the user which configurations are the problem.
						 * Demonstrates how to use the VvcNotificationCenter to
						 * easily display HTML elements. 
						 */
						if(notification){
							notification.close();
						}
						
						var args = {
							ioArgs: err.response
						};
						
						if(args.ioArgs){
							ncFactory.getRdfParts(args);
							var message = args.title;
							
							var type;
							var resources;
							
							var description;
							var solution;
							
							var typeTitle = "Archiving exception:";
							var resourcesTitle = "Resources:";
							
							var typeIndex = message.indexOf(typeTitle);
							var resourcesIndex = message.indexOf(resourcesTitle);

							type = message.substring(typeIndex + typeTitle.length, 
									resourcesIndex >= 0 ? resourcesIndex : message.length).trim();
							
							var messageId = "error_" + type;;
							
							if(resourcesIndex >= 0){
								var resourcesAsString =  message.substring(resourcesIndex + resourcesTitle.length, 
										message.length).trim();
								resources = resourcesAsString.split(',');
								description = dojo.string.substitute(messages[messageId + ".explanation"], [resources.length]);
								
								var blockingResources = dojo.create("span");
								var list = dojo.create("textarea", {
									style: "width: 850px; height: 100px",
									readonly: true
								});
								dojo.forEach(resources, dojo.hitch(this, function(resource){
									list.appendChild(dojo.doc.createTextNode(resource + "\n"));
								}));
								blockingResources.appendChild(dojo.doc.createTextNode(description));
								blockingResources.appendChild(dojo.create("br"));
								blockingResources.appendChild(list);
								description = blockingResources.outerHTML;
							}

							if (messages[messageId]) {
								ncFactory.createError({
									"title": dojo.string.substitute(messages[messageId], {
										operation: data.archive ? messages.restore_operation_title : messages.archive_operation_title,
										title: data.title, 
										type: data.type
									}),
									"description": description,
									"solution": messages[messageId + ".useraction"],
								}).dispatch();
							} else {
								ncFactory.createXhrError({
									title: data.archive ? messages.unarchiveConfigurationErrorTitle : messages.archiveConfigurationErrorTitle,
									xhrError: args.ioArgs.xhr, 
									ioArgs: args.ioArgs}
								).dispatch();
							}
						} else {
							ncFactory.createXhrError({
								title: data.archive ? messages.unarchiveConfigurationErrorTitle : messages.archiveConfigurationErrorTitle,
								xhrError: args.xhr, 
								ioArgs: err.response}
							).dispatch();
						}
					});
					if(options && !options.supressNotifications){
						var title = data.archive ? messages.unarchvingConfigurationNotificationTitle : messages.archivingConfigurationNotificationTitle; 
						var description = data.archive ? messages.unarchivingConfigurationNotificationDescription : messages.archivingConfigurationNotificationDescription;
						notification = ncFactory.createInfo({
							"title" : title,
							"description" : description
						}).dispatch();
					}
				})
			};

			var applicable = function(button){
				if(data && data.underConstruction){
					return false;
				}
				var applicable = data.allowedActions[SecurityConstants.permissions.archive_restore];
				if(!applicable) {
					button.set("title", SecurityServiceUtils.getNoPermissionMessage(data.controllingAreaName));
				}
				return applicable;
			};
			return this._makeButton(args, options, applicable);
		},
		
		getProjectAssociationButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-associate-project-or-team-area",
				title: messages.associateProjectOrTeamAreaAction,
				onClick: dojo.hitch(this, function(){
					InfoContentUtils.showDialog(com.ibm.team.vvc.shared.web.ui.util.TeamAreaDialogContent, messages.associateProjectOrTeamAreaDialog, {
						uri: data.uri, 
						spaceUri: data.spaceUri, 
						title: data.title,
						okFunction: dojo.hitch(this, function(projectOrTeamAreaUri, type) {
							var deferred = AccessUtils.setConfigurationOwner(data.uri, type[0] === "projectArea" ? null : projectOrTeamAreaUri);
							deferred.addCallback(this, function (param) {
								if(options && options.actionCallback){
									options.actionCallback(param);
								}
							});
							return deferred;
						})
					});
				})
			}
			var applicable = function(button){
				if(data && data.underConstruction){
					return false;
				}
				if(!data.archive && data.mutable == true){
					return data.allowedActions[SecurityConstants.permissions.setControlledBy];
				}
				return false;
			}
			return this._makeButton(args, options, applicable);
		},
		
		getCheckboxButton: function(data, options, onClickArgs){
			var checked = false;
			var shaded = false;		
			var args = {
				imgClass: "icons-checkbox-unchecked",
				title: messages.selectToCompareOrDeliver,
				onClick: dojo.hitch(this, function(){
					checked = !checked;
					var imageClass = "icons-checkbox-" + (checked ? "" : "un") + "checked" + (shaded ? "-shaded" : "");
					button.set("imgClass", imageClass);
					if(options && options.actionCallback){
						options.actionCallback({
							"checked" : checked
						});
					}
				})
			}
			var applicable = function(button){
				if(data && data.underConstruction){
					return false;
				}
				return true;
			};
			var button = this._makeButton(args, options, applicable);
			return button;
		},
		
		getRefreshButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-refresh",
				title: messages.refreshButtonTitle,
				onClick: dojo.hitch(this, function(){
					if(options && options.actionCallback){
						options.actionCallback();
					}
				})
			};
			var applicable = function(button){
				return true;
			};
			var button = this._makeButton(args, options, applicable);
			return button;
		},
			
		getExpandAllButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-expand-all",
				title: messages.expandAllButtonTitle,
				onClick: dojo.hitch(this, function(){
					if(options && options.actionCallback){
						options.actionCallback();
					}
				})
			};
			var applicable = function(button){
				return true;
			};
			var button = this._makeButton(args, options, applicable);
			return button;
		},
		
		getCollapseAllButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-collapse-all",
				title: messages.collapseAllButtonTitle,
				onClick: dojo.hitch(this, function(){
					if(options && options.actionCallback){
						options.actionCallback();
					}
				})
			};
			var applicable = function(button){
				return true && !options.disabled === true;
			};
			var button = this._makeButton(args, options, applicable);
			return button;
		},
		
		getShowArchivedButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-show-archived",
				title: messages.showArchivedButtonTitle,
				onClick: dojo.hitch(this, function(){
					button.set("title", button.pressed ? messages.hideArchivedButtonTitle : messages.showArchivedButtonTitle);
					if(options && options.actionCallback){
						options.actionCallback();
					}
				})
			};
			var applicable = function(button){
				return true && !options.disabled === true;
			};
			var button = this._make2StateButton(args, options, applicable);
			return button;
		},
		
//TODO: This will be used for setting master VVC
//		getTemporarySetMasterVVCButton: function(onClick){
//			var args = {
//				imgClass: "icons-create-workspace",
//				title: "Temporary action to set master VVC",
//				onClick: dojo.hitch(this, function(){
//					var master = window.prompt("Enter master VVC server (may be empty if this is the master):");
//					
//					//empty string ok
//					if (master != null && master != undefined) {
//						var args = {
//							url: VvcUtils.getVvcUri() + "/globalConfigurations/setMaster?master=" + encodeURIComponent(master),
//						};
//						jazz.client.xhrPost(args);						
//					}
//				})
//			};
//			return this._makeButton(args, {} /*options*/, function() { return true; });
//		},		
					
		getCreateChangesetButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-changeset-create ",
				title: messages.createChangeset,
				onClick: dojo.hitch(this, function(){				
					dojo.require("com.ibm.team.vvc.shared.web.ui.changesets.CreateChangesetForm");
					var CreateChangesetForm = com.ibm.team.vvc.shared.web.ui.changesets.CreateChangesetForm;
					dojo.ready(dojo.hitch(this, function(){					
						var form = new CreateChangesetForm({
							configuration: ContextUtils.getContextUri()
						});
						
						form.show();
						
						// Not great.  Need to revisit how this form works later...
						var handle = dojo.subscribe("vvc.changeset_created", dojo.hitch(this, function(args){
							dojo.unsubscribe(handle);
							if(options && options.actionCallback){
								options.actionCallback();
							}
						}));
					}));
				})
			}
			
			var applicable = function(button){
				return true;
			};
			
			var button = this._makeButton(args, options, applicable);
			return button;
		},
		
		getShareChangesetButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-changeset-share",
				title: messages.shareChangeset,
				onClick: dojo.hitch(this, function(){
					var def = jazz.client.xhrPost({
						handleAs: 'text',
						headers: {
							"Content-Type" : "text/plain",
							"VvcChangeSetTargetWorkspace" : ContextUtils.getContextUri()
						},
						url: data.uri,
						postData: JSON.stringify({
							action: "COMMIT" 
						})
					});
					
					def.then(dojo.hitch(this, function(){
						ncFactory.createSuccess({
							title: messages.shareSuccessTitle,
							description: dojo.string.substitute(messages.shareSuccessDescription, {changesetTitle: data.title}),
						}).dispatch();
						
						if(options && options.actionCallback){
							options.actionCallback();
						}
					}), dojo.hitch(this, function(error){
						ncFactory.createError({
							title: dojo.string.substitute(configurationContextMessages["failedToCommitChangeset"], [data.title]),
							description: configurationContextMessages["failedToCommitChangeset.explanation"],
							solution: configurationContextMessages["failedToCommitChangeset.useraction"]
						}).dispatch();
					}));
				})
			}
			
			var applicable = function(button){
				return "committed" !== data.state.toLowerCase();
			};
			
			var button = this._makeButton(args, options, applicable);
			return button;
		},
		
		getDiscardChangesetButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-changeset-discard",
				title: messages.discardChangeset,
				onClick: dojo.hitch(this, function(){
					InfoContentUtils.showQuestionDialog(
						configurationContextMessages.discardConfirmationTitle,
						dojo.string.substitute(configurationContextMessages.discardConfirmationContent, {changesetTitle: data.title}),
						configurationContextMessages.discardConfirmationOkButton,
						configurationContextMessages.cancelButton,
						dojo.hitch(this, function(){
							var deferred = jazz.client.xhrDelete({
								handleAs: 'text',
								url: data.uri,
							});
							
							deferred.then(dojo.hitch(this, function(response){
								ncFactory.createSuccess({
									title: messages.discardSuccessTitle
								}).dispatch();
								
								if(options && options.actionCallback){
									options.actionCallback();
								}
							}), dojo.hitch(this, function(error){
								ncFactory.createError({
									title: dojo.string.substitute(configurationContextMessages["failedToDiscardChangeset"], [data.title]),
									description: configurationContextMessages["failedToDiscardChangeset.explanation"],
									solution: configurationContextMessages["failedToDiscardChangeset.useraction"]		
								}).dispatch();							
							}));
						})
					);
				})
			}
			
			var applicable = function(button){
				return "committed" !== data.state.toLowerCase();
			};
			
			var button = this._makeButton(args, options, applicable);
			return button;
		},
		
		getCompleteChangesetButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-changeset-complete",
				title: messages.completeChangeset,
				onClick: dojo.hitch(this, function(){
					InfoContentUtils.showQuestionDialog(
						configurationContextMessages.completeConfirmationTitle,
						dojo.string.substitute(configurationContextMessages.completeConfirmationContent, {changesetTitle: data.title}),
						configurationContextMessages.completeConfirmationOkButton,
						configurationContextMessages.cancelButton,
						dojo.hitch(this, function(){
							var def = jazz.client.xhrPost({
								handleAs: 'text',
								headers: {
									"Content-Type" : "text/plain"
								},
								url: data.uri,
								postData: JSON.stringify({ 
									action: "COMPLETE" 
								})
							});
							
							def.then(dojo.hitch(this, function(results){
								ncFactory.createSuccess({
									title: messages.completeSuccessTitle,
									description: dojo.string.substitute(messages.completeSuccessDescription, {changesetTitle: data.title}),
								}).dispatch();
								
								if(options && options.actionCallback){
									options.actionCallback();
								}
							}), dojo.hitch(this, function(error){
								ncFactory.createError({
									title: dojo.string.substitute(configurationContextMessages["failedToCompleteChangeset"], [data.title]),
									description: configurationContextMessages["failedToCompleteChangeset.explanation"],
									solution: configurationContextMessages["failedToCompleteChangeset.useraction"]		
								}).dispatch();
							}));
						})
					);
				})
			}
			
			var applicable = function(button){
				return "committed" !== data.state.toLowerCase();
			};
			
			var button = this._makeButton(args, options, applicable);
			return button;
		},
		
		getReverseChangesetButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-changeset-reverse",
				title: messages.reverseChangeset,
				onClick: dojo.hitch(this, function(){							
					InfoContentUtils.showQuestionDialog(
						messages.reverseChangesetTitle,
						dojo.string.substitute(messages.reverseChangesetContent, {changesetTitle: data.title}),
						messages.reverseChangesetOkButton,
						messages.cancelButton,
						dojo.hitch(this, function(){
							var def = jazz.client.xhrPost({
								handleAs: 'text',
								headers: {
									"Content-Type" : "text/plain",
									"VvcChangeSetTargetWorkspace" : ContextUtils.getContextUri()
								},
								url: data.uri,
								postData: JSON.stringify({ 
									action: "REVERSE", 
								})
							});
							
							var tempContext = {
								closeEvent: dojo.hitch(this, function(){
									if(options && options.actionCallback){
										options.actionCallback();
									}
								})
							};
							var notification = ncFactory.createInfo({
								title: messages.initializingReverseChangeset,
								closeOnEvent: {
									context: tempContext,
									event: "closeEvent"
								}
							}).dispatch();
							
							def.then(dojo.hitch(this, function(results){
								tempContext.closeEvent();
							}), dojo.hitch(this, function(error){
								notification.close();
								ncFactory.createXhrError({xhrError: error, ioArgs: error.response.options.ioArgs}).dispatch();
							}));
						})
					);
				})
			}
			
			var applicable = function(button){
				return "committed" === data.state.toLowerCase();
			};
			
			var button = this._makeButton(args, options, applicable);
			return button;
		},
		
		getRenameChangesetButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-changeset-rename",
				title: messages.renameChangeset,
				onClick: dojo.hitch(this, function(){
					// TODO: Make this not a prompt...
					var oldName = data.title; 
					var name = prompt("rename", data.title)
									
					if(!name || name.length == 0){
						// Show warning?
						return;
					}
					
					name = name.trim();
															
					var def = jazz.client.xhrPost({
						handleAs: 'text',
						headers: {
							"Content-Type": "text/plain"
						},
						url: data.uri,
						postData: JSON.stringify({
							action: "RENAME",
							VvcChangesetRenameNewName: name
						})
					});
					
					def.then(dojo.hitch(this, function(){
						ncFactory.createSuccess({
							title: messages.renameSuccessTitle,
							description: dojo.string.substitute(messages.renameSuccessDescription, {
								oldChangesetTitle: oldName,
								newChangesetTitle: name
							})
						}).dispatch();
						
						if(options && options.actionCallback){
							options.actionCallback();
						}
					}), dojo.hitch(this, function(error){
						ncFactory.createError({
							title: dojo.string.substitute(messages["failedToRenameChangeset"], [data.title]),
							description: messages["failedToRenameChangeset.explanation"],
							solution: messages["failedToRenameChangeset.useraction"]
						}).dispatch();
					}));
				})
			}
			
			var applicable = function(button){
				return true;
			};
			
			var button = this._makeButton(args, options, applicable);
			return button;
		},
		
		getLinkChangeRequestToChangesetButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-changeset-workitem",
				title: messages.linkChangeset,
				onClick: dojo.hitch(this, function(){
					var def = new dojo.Deferred();					
					LinkToWorkItemUtil.vvcLinkToWorkItemAction(data.uri, def);
					
					def.then(dojo.hitch(this, function(results){
						if(options && options.actionCallback){
							options.actionCallback();
						}
					}), dojo.hitch(this, function(error){
						ncFactory.createXhrError({xhrError: error, ioArgs: error.response.options.ioArgs}).dispatch();
					}));
				})
			}
			
			var applicable = function(button){
				return true;
			};
			
			var button = this._makeButton(args, options, applicable);
			return button;
		},

		/*
		 * See also: ContextUtils.buildHistoryButtonForResource
		 */
		getShowHistoryButton: function(data, options, onClickArgs){
			var urlParams = {
				conceptUri: data.conceptUri,
				contextUri: data.contextUri ? data.contextUri : ContextUtils.getContextUri(),
				resourceDisplayName: data.displayName ? data.displayName : ""
			};
			
			var args = {
				imgClass: "icons-show-history",
				title: "Show history",
				href: VvcUtils.getVvcUri() + "/web?" + dojo.objectToQuery(urlParams) + "#action=exploreHistory"
			}
			
			
			if(options && options.actionCallback){
				console.warn("actionCallback not supported for showHistoryButton and will be ignored");
			}
				
			var applicable = function(button){
				return true;
			};
			
			var button = this._makeButton(args, options, applicable);
			return button;
		},
		
		getToggleChangesetsShowUserOrShowAllButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-all-users",
				title: messages.showAllTeamChangesets,
				onClick: dojo.hitch(this, function(){
					button.set("title", button.pressed ? messages.showUsersChangesets: messages.showAllTeamChangesets);
					if(options && options.actionCallback){
						options.actionCallback();
					}
				})
			};
			var applicable = function(button){
				return true && !options.disabled === true;
			};
			var button = this._make2StateButton(args, options, applicable);
			return button;
		},
		
		getToggleShowOnlyActiveChangesetsButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-show-active-changesets",
				title: messages.showAllChangesets,
				onClick: dojo.hitch(this, function(){
					button.set("title", button.pressed ? messages.showActiveChangesets: messages.showAllChangesets);
					if(options && options.actionCallback){
						options.actionCallback();
					}
				})
			};
			var applicable = function(button){
				return true && !options.disabled === true;
			};
			var button = this._make2StateButton(args, options, applicable);
			return button;
		},
		
/* ----------------------------------------------------------------------------------------------------- */
// Button helpers
		_makeButton: function(args, options, applicable){
			// Force enabled to be false, wait for the isApplicable code to run
			args.enabled = false;
			var button = new jazz.ui.toolbar.Button(args);
			var isApplicable = dojo.hitch(this, applicable, button);
			dojo.when(isApplicable(), dojo.hitch(this, function(isEnabled){
				// Respect the explicit disablement of a button, then respect the buttons own enablement.
				button.set("enabled", options && options.disabled != undefined ? options.disabled === false : true && isEnabled);
			}));
			return button;
		},
		
		_make2StateButton: function(args, options, applicable){
			// Force enabled to be false, wait for the isApplicable code to run
			args.enabled = false;
			var button = new jazz.ui.toolbar.TwoStateButton(args);
			var isApplicable = dojo.hitch(this, applicable, button);
			dojo.when(isApplicable(), dojo.hitch(this, function(isEnabled){
				// Respect the explicit disablement of a button, then respect the buttons own enablement.
				button.set("enabled", options && options.disabled != undefined ? options.disabled === false : true && isEnabled);
			}));
			return button;
		}
		
/* ----------------------------------------------------------------------------------------------------- */
		
}
})();
